import numpy as np

def stellar_density(P, jeff):

   '''
   P = planet orbital period in days
   jeff = ratio of planet semi-major axis to stellar radius
   '''

   G = 6.67e-11 #[m^3/(kg*s^2)]

   print('The density of your star is {} g/cm^3. Sweet.'.format(np.round(((3*np.pi/G) * (1/(P*60*60*24)**2) * (jeff)**3)/1000,2)))


kepler_offset = 2454833.0
tess_offset = 2457000.0

def bjd2bkjd(t):
    '''
    Convert a
    BJD (Barycentric Julian Date)
    into a
    BKJD (Barycentric Kepler Julian Date)
    via the relationship
    BKJD = BJD - 2454833.0

    Parameters
    ----------
    t : float, array
        Time in BJD (Barycentric Julian Date), with values like 2454123.123.

    Returns
    -------
    t_kepler : float, array
        Time in BKJD (Barycentric Kepler Julian Date).
    '''
    return t - kepler_offset


def bkjd2bjd(t_kepler):
    '''
    Convert a
    BKJD (Barycentric Kepler Julian Date)
    into a
    BJD (Barycentric Julian Date)
    via the relationship
    BKJD = BJD - 2454833.0

    Parameters
    ----------
    t_kepler : float, array
        Time in BKJD (Barycentric Kepler Julian Date)

    Returns
    -------
    t : float, array
        Time in BJD (Barycentric Julian Date), with values like 2454123.123.
    '''
    return t_kepler + kepler_offset

def bjd2btjd(t):
    '''
    Convert a
    BJD (Barycentric Julian Date)
    into a
    BTJD (Barycentric TESS Julian Date)
    via the relationship
    BTJD = BJD - 2457000.0

    Parameters
    ----------
    t : float, array
        Time in BJD (Barycentric Julian Date), with values like 2454123.123.

    Returns
    -------
    t_tess : float, array
        Time in BTJD (Barycentric TESS Julian Date)
    '''
    return t - tess_offset


def btjd2bjd(t_tess):
    '''
    Convert a
    BTJD (Barycentric TESS Julian Date)
    into a
    BJD (Barycentric Julian Date)
    via the relationship
    BTJD = BJD - 2457000.0

    Parameters
    ----------
    t_tess : float, array
        Time in BTJD (Barycentric TESS Julian Date)

    Returns
    -------
    t : float, array
        Time in BJD (Barycentric Julian Date), with values like 2454123.123.
    '''
    return t_tess + tess_offset

def find_appropriate_epoch(lc, t0=0):
    '''
    For a given light curve object, convert a transit epoch (in BJD, with
    values like 2454123.123) into the right values to match the light curve's
    time_format. This handles the offsets between BKJD (Kepler) and BTJD (TESS)
    and BJD (normal Barycentric Julian Dates).

    Parameters
    ----------

    lc : lightkurve.lightcurve.LightCurve
        This is any light curve object.

    t0 : float
        This is a single time of mid-transit, expressed in
        Barycentric Julian Date (BJD) with values like 2454123.123.
    '''

    # choose the appropriate offset for the lc's time format
    if lc.time_format == 'bkjd':
        epoch = bjd2bkjd(t0)
    elif lc.time_format == 'btjd':
        epoch = bjd2btjd(t0)
    else:
        epoch = t0

    # return the epoch
    return epoch



from .imports import *

def decide_writer(filename, **kw):
    '''
    Decide which animation writer to use, given a desired output filename.

    Parameters
    ----------

    '''
    # figure out the writers to use
    if '.gif' in filename:
        try:
            writer = ani.writers['pillow'](**kw)
        except (RuntimeError, KeyError):
            writer = ani.writers['imagemagick'](**kw)
        except:
            raise RuntimeError('This python seems unable to make an animated gif.')
    else:
        try:
            writer = ani.writers['ffmpeg'](**kw)
        except (RuntimeError,KeyError):
            raise RuntimeError('This computer seems unable to ffmpeg.')
    return writer

def mkdir(path):
    '''
    Make a directory. Don't complain if the directory already exists.

    Parameters
    ----------
    path : str
        The path of the directory to make.
    '''
    try:
        os.mkdir(path)
        print("made {}".format(path))
    except:
        pass



'''
Define an Aperture object, which
    -has a location
    -does photometry
    -can be plotted and picked
    -can be unplotted
'''

from ..imports import *
import photutils
from astropy.stats import sigma_clipped_stats
from astropy.table import Table


class InteractiveAperture(photutils.CircularAperture):

    def __init__(self, name='', pos=(0,0), loupe=None,  aperture_radius=5, background_radii=[15, 25], subtract_background=False):
        '''
        Initialize an interactive aperture.

        Parameters
        ----------
        name : str
            A simple name for the aperture
        pos : tuple
            A two-element list of (x,y)? pixel positions for the center
        loupe : a loupe.Loupe object
            A organizing Loupe in which this aperture is embedded
        aperture_radius : float
            A radius for the aperture
        background_radii : list of two floats
            Inner and outer radii for a background-estimation annulus
        subtract_background : bool
            Should we do background subtraction?
        '''

        # initialize the CircularAperture as normal
        photutils.CircularAperture.__init__(self, pos, aperture_radius)

        # create a background annulus
        r_in, r_out = background_radii
        r_out = np.maximum(r_out, r_in + 1)
        self.background_aperture = photutils.CircularAnnulus(pos, r_in, r_out)
        self.subtract_background = subtract_background

        # keep track of this aperture's name
        self.name = name

        # keep track of the loupe this aperture is a part of
        self.loupe = loupe

        # keep track of the flux in this aperture with an undefined flux
        self.flux = np.nan

    def plot(self, ax):
        '''
        Plot this aperture onto the given axes.

        Parameters
        ----------
        ax : matplotlib axes
            The axes in which we shoudl plot this aperture
        '''

        # create an empty dictionary to store any plotted elements
        self.plotted = {}

        # (this can handle only single-position apertures)
        pos = self.positions[0]

        # (pull out the apeture radius)
        aperture_radius = self.r

        # create and draw the circle for the aperture
        self.plotted['circle'] = plt.Circle(pos, aperture_radius, color='black',fill=False,zorder=100,linewidth=2, picker=True)
        ax.add_artist(self.plotted['circle'])

        # put a crosshair at the center of the aperture
        self.plotted['marker'] = plt.scatter(pos[0], pos[1], marker='+', alpha=0.3, color='black')

        # add a label next to the aperture
        self.plotted['label'] = plt.text(pos[0]+aperture_radius+1, pos[1], self.name, va='center', zorder=11, color='black')

        # create and draw the background annulus around the aperture
        r_in, r_out = self.background_aperture.r_in, self.background_aperture.r_out
        bgkw = dict(color='black', alpha=0.3, fill=False, zorder=100)
        self.plotted['background_inner_circle'] = plt.Circle(pos, r_in, **bgkw)
        self.plotted['background_outer_circle'] = plt.Circle(pos, r_out, **bgkw)

        # hide the background annulus if we're not subtracting the background
        for k in ['background_inner_circle', 'background_outer_circle']:
            ax.add_artist(self.plotted[k])
            self.plotted[k].set_visible(self.subtract_background)

    def erase(self):
        '''
        Remove any traces of this aperture from the plot.
        '''

        # loop through all plotted elements and remove them
        for k in self.plotted.keys():
            self.plotted[k].remove()

    def update(self, plot=True):
        '''
        After a plot has already been created, update its hidden properties,
        as well as its appearance on the interactive plot. This function
        is responsible for performing photometry on a particular frame.
        This function pulls updated properties from the underlying Loupe.

        Parameters
        ----------
        plot : bool
            Should we update the plot?
        '''

        # update the various radii and options
        self.r = self.loupe.aperture_radius
        self.background_aperture.r_in, self.background_aperture.r_out = self.loupe.background_radii
        self.subtract_background = self.loupe.subtract_background


        # update the plot (if we're supposed to)
        if plot:
            pos = self.positions[0]
            self.plotted['circle'].set_radius(self.r)
            self.plotted['label'].set_position((pos[0]+self.r+1, pos[1]))
            self.plotted['marker'].set_offsets(pos)
            self.plotted['background_inner_circle'].set_radius(self.background_aperture.r_in)
            self.plotted['background_outer_circle'].set_radius(self.background_aperture.r_out)
            for k in ['background_inner_circle', 'background_outer_circle']:
                self.plotted[k].set_visible(self.subtract_background)

        # create a table of measurements
        self.table = Table(data=[[self.name], [pos[0]], [pos[1]], [0.0]], names=['#','x', 'y', 'flux'])

        # do the photometry in the aperture
        aperture, _ = self.do_photometry(self.loupe.image)

        # subtract the background, if we're supposed to
        if self.subtract_background:

            # calculate the background flux
            background_mask = self.background_aperture.to_mask(method='center')[0]
            background_masked_image = background_mask.multiply(self.loupe.image)
            values = background_masked_image[background_masked_image != 0]
            _, background_median, _ = sigma_clipped_stats(values)
            aperture_background = background_median*self.area()
            aperture -= aperture_background
            self.table['sky'] = aperture_background
            self.table['sky'].format = '.2e'

        # store our final flux in the table
        self.table['flux'] = aperture

        # update the default format for the table outputs
        self.table['x'].format = '.0f'
        self.table['y'].format = '.0f'
        self.table['flux'].format = '.2e'

    #def __str__(self):
    #    '''
    #    How should this aperature appear as a string (for example, if it is
    #    included in a `'{}'.format()` statement)?
    #    '''
    #    #return '{:>8} at ({:4.1f}, {:4.1f}) sums to {:10.1f} (area of {:.1f} pixels).'.format(self.name, self.positions[0][0], self.positions[0][1],  self.flux, self.area())
    #    return repr(self.table)

#class ApertureWithBackground(InteractiveAperture):
#    pass
    #def __init__(self, name='', pos=(0,0), aperture_radius=3 ):
    #    pass



from astropy.modeling import models, fitting, optimizers, statistic, custom_model
import numpy as np
import emcee
import batman
import corner as triangle
import matplotlib.pyplot as plt

"""

This script takes in an astropy transit model and a lightkurve object,
determines which parameters are variable, and runs an MCMC
to fit those parameters and create a maximum likelihood model.

"""

def lnprob(params, astropy_model, lc):

    """
    Determine which parameters are variable, and set their value to the
    current parameter value in the chain.
    -----------
    """
    n = len(astropy_model.param_names)

    i = 0
    for k in astropy_model.param_names:
        if astropy_model.fixed[k] == False:
            for m in range(n):
                if astropy_model.param_names[m] == str(k):
                    astropy_model.parameters[m] = params[i]
                    i += 1

    if (astropy_model.a <= astropy_model.b):
        astropy_model.a = astropy_model.b + 0.01

    """
    Create the new model and calculate the log probability for a Gaussian
    likelihood.
    ------------
    """
    model = astropy_model(lc.time)

    if (0.0 <= astropy_model.radius <= 1.0) and (lc.time[0] <= astropy_model.t0 <= lc.time[-1] ) and (1.0 <= astropy_model.a <= 200.0 ):
        chisq = np.nansum((lc.flux - model)**2/(lc.flux_err)**2)
        lnp = np.nansum(1/np.sqrt(2*np.pi*(lc.flux_err))) - 0.5*chisq

        return lnp

    return -np.inf

def mcmc_fit(astropy_model, lc, nsteps = 10000, saveplots=False):

    '''
    This function will employ a Markov-Chain Monte Carlo to fit any number
    of free parameters to an exoplanet transit light curve.


    Parameters
    ----------

    lc: LightCurve object
        This is a `lightkurve`-style LightCurve object, which contains
        the attributes `lc.time` (times in JD) and `lc.flux` (the brightness
        of the star), as well as lots of methods for analysis and plotting.

    astropy_model: An 'astropy model' object
        This model obect contains all the model parameters, their names,
        and their bounds if they are free parameters. It interfaces
        with a custom model to generate the function of interest,
        which is a BATMAN light curve model in this case.


    Returns
    -------

    max_likelihood: A dictionary
        This dictionary contains the names of the variable parameters,
        as well as a 3-element list of the maximum likelihood value,
        and +/- 1 sigma uncertainties.

    sampler: EnsembleSampler Object
        This object has several capabilities. In this MCMC, it is primarily
        used to run the emcee function and give us access to sample parameter
        values generated by the Monte Carlo.

    '''

    """
    First, check that the lc object is normalized and folded. If not,
    fold and normalize it.
    ----------
    """

    fold_check = input(["Is your light curve folded? (y/n)"])

    if fold_check == "n":
        lc = lc.fold(period = astropy_model.period.value)
        print("That's okay, I did it for you")

    norm_check = input(["Is your light curve normalized? (y/n)"])

    if norm_check == "n":
        lc = lc.normalize()
        print("Do I have to do everything?")

    """
    Determine the names of all variable parameters, so that we can use
    these names later.
    ----------
    """

    variable_names = []
    i = 0
    for k in astropy_model.param_names:
        if astropy_model.fixed[k] == False:
            variable_names.append(str(k))
            i += 1

    """
    Create an organized parameter dictionary so that we can easily
    populate some initial parameter values, and then define an array
    of these initial values, to be input into emcee.
    ----------
    """

    organized = dict(period = astropy_model.period, t0 = astropy_model.t0,
                 radius = astropy_model.radius, a = astropy_model.a,
                 b = astropy_model.b, baseline = astropy_model.baseline,
                 ld1 = astropy_model.ld1, ld2 = astropy_model.ld2)

    ndim, nwalkers, nsteps = i, 100, nsteps
    burnin = int(0.2*nsteps)

    param_initial = []
    for k in astropy_model.param_names:
        if organized[k].fixed == False:
            bounds = organized[k].bounds
            organized[k].initialwalkers = np.random.uniform(bounds[0],bounds[1],nwalkers)
            param_initial.append(organized[k].initialwalkers)

    p0 = np.transpose(param_initial)

    """
    Create a sampler object and run the MCMC.
    ----------
    """

    sampler = emcee.EnsembleSampler(nwalkers, ndim, lnprob, args=[astropy_model, lc])
    result = sampler.run_mcmc(p0, nsteps)

    samples = sampler.chain[:, burnin:, :].reshape((-1, ndim))

    """
    Set up the max_likelihood dictionary and populate it with the sample
    results. Then, plot each free parameter value as a function of the
    step in the chain each walker is at. This will show us whether or not
    parameters are converging, and at which values.
    ----------
    """

    max_likelihood = {}

    fig,ax = plt.subplots(i, 1, sharex=True, figsize=(8, 9))
    for j in range(i):
        param_samples = sampler.flatchain[int(nsteps*nwalkers/i):,j]
        max_likelihood[variable_names[j]] = np.percentile(param_samples, [16., 50., 84.])

        ax[j].plot(sampler.chain[:, :, j].T, color="k", alpha=0.4)
        ax[j].set_ylabel(variable_names[j])
        if j == i-1:
            ax[j].set_xlabel('Steps')
    if saveplots == True:
        plt.savefig('walker_plot.pdf', clobber=True)

    """
    Set up a corner plot, which will show us contour plots for each pair
    of parameters.
    ----------
    """

    ndim = i
    labels = variable_names

    fig, axes = plt.subplots(ndim, ndim, figsize=(13,13))
    triangle.corner(samples, bins=20, labels=labels,
                      max_n_ticks=3,plot_contours=True,quantiles=[0.16,0.5,0.84],fig=fig,
                      show_titles=True,verbose=True,range=None)

    if saveplots == True:
        plt.savefig('corner_plot.pdf', clobber=True)

    """
    Finally, set the model parameters to the maximum likelihood values and
    produce a plot of the data with that best fit model.
    ----------
    """

    n = len(astropy_model.param_names)
    i = 0
    for k in astropy_model.param_names:
        if astropy_model.fixed[k] == False:
            for m in range(n):
                if astropy_model.param_names[m] == str(k):
                    astropy_model.parameters[m] = max_likelihood[k][1]
                    i += 1

    plt.figure()
    lc.errorbar(alpha= 0.5,zorder=0,label='Data')
    plt.plot(lc.time,astropy_model(lc.time),zorder=100,
                label='Maximum Likelihood Model',
                color='b')
    plt.title('Light Curve with Maximum Likelihood Model')
    plt.legend()

    if saveplots == True:
        plt.savefig('Best-fit-model.pdf',clobber=True)

    return max_likelihood,sampler



import numpy as np, matplotlib.pyplot as plt
from lightkurve import LightCurve
plt.ion()

def create_photon_lightcurve(N=100, cadence=0.5/24.0, duration=1):
    '''
    Create a lightkurve object, assuming we detect
    a particular number of photons per exposure.

    Parameters
    ----------

    N: float
        The average number of photons per exposure.
    cadence: float
        The time between successive exposures (in days).
    duration: float
        The total time spanned by the light curve (in days).
    '''
    t = np.arange(0, duration, step=cadence)
    photons = np.random.poisson(N, t.shape)
    lc = LightCurve(time=t,
                    flux=photons,
                    flux_err=np.sqrt(photons),
                    meta=dict(name='Poisson light curve with {} photons/s'.format(N)))
    return lc

def create_approximate_lightcurve(N=100, cadence=0.5/24.0, duration=1):
    '''
    Create a lightkurve object, assuming we detect
    a particular number of photons per exposure,
    but treating the PDF for photon counts with the
    Gaussian approximation, instead of a true Poisson
    distribution.

    Parameters
    ----------

    N: float
        The average number of photons per exposure.
    cadence: float
        The time between successive exposures (in days).
    duration: float
        The total time spanned by the light curve (in days).
    '''
    t = np.arange(0, duration, step=cadence)
    photons = np.random.normal(N, np.sqrt(N), t.shape)
    lc = LightCurve(time=t,
                    flux=photons,
                    flux_err=np.sqrt(photons),
                    meta=dict(name='Poisson light curve with {} photons/s'.format(N)))
    return lc

def catch_photons_in_bucket(rate=1, diameter=1.0, time=60*30.0, visualize=True):
    '''
    Simulate a telescope catching photons,
    for a given photon flux (photons/s/m**2),
    telescope diameter (m), and exposure time (s).

    Parameters
    ----------
    rate: float
        The rate at which photons are coming towards us,
        expressed in units of (photons/s/m**2).

    diameter: float
        The effective diameter of a circular telescope,
        expressed in units of (m).

    time: float
        The exposure duration, expressed in units of (s).

    Returns
    -------
    N: int
        The number of photons that land in the telescope
    '''

    # what's the radius of the telescope
    radius = diameter/2.0

    if visualize:

        # make a rectangle that at least include the telescope
        square = np.maximum(1.3*diameter, 1.0)

        # what's the expected total number of photons
        area = square*square

        # create a rectangle of randomly located photons
        Ntotal = np.random.poisson(area*time*rate)
        x, y = np.random.uniform(-square/2, square/2, (2, Ntotal))

        # determine which of these photons landed in the telescope
        incircle = (x**2 + y**2) < radius**2
        outofcircle = ~incircle
        N = np.sum(incircle)

        kw = dict(marker='.', markeredgecolor='none', alpha=0.5, linewidth=0)
        plt.figure(figsize=(4,4))
        plt.plot(x[incircle], y[incircle], color='mediumseagreen', **kw)
        plt.plot(x[outofcircle], y[outofcircle], color='black', **kw)

        # draw a circle
        theta = np.linspace(0, 2*np.pi, 1000)
        plt.plot(radius*np.sin(theta), radius*np.cos(theta), linewidth=4, color='black')

        # label the radius of the circle
        plt.text(0, -radius*1.1, '{}m'.format(diameter), ha='center', va='top', fontweight='bold', fontsize=10, color='mediumseagreen')

        # add a title
        plt.title('{} photons gathered in {}s\n({} photons/s/m$^2$)'.format(N, time, rate))

        # set the aspect ratio of the plot to 1:1
        plt.axis('equal')
        plt.xlim(-square/2, square/2)
        plt.ylim(-square/2, square/2)

        # get rid of the square around the plot
        plt.axis('off')
        plt.subplots_adjust(top=0.8)
        plt.show()

    else:

        # if we don't need to make plot, just draw a Poisson number
        area = np.pi*radius**2
        N = np.random.poisson(area*time*rate)

    return N



from .test_lightcurves import *
from .test_statistics import *
from .test_models import *
from .test_fitting import *
from .test_tools import *
from .test_photometry import *
from .test_tpf import *
from .test_imaging import *
from .test_photometry import *



'''
henrietta is a set of tools for playing with photometric light curves
in the Fall 2018 session of ASTR3400: Research Methods in Astronomy,
taught at CU Boulder by Zach Berta-Thompson (https://github.com/zkbt/).
'''

def whoisthisawesomelady():

    print('''
    Henrietta Swan Leavitt (1868-1921) was an astronomer at the Harvard College
    Observatory. Her pioneering work in the study of variable stars allowed
    scientists to measure the size of the Universe for the first time.

    If you'd like to learn more about her, check out her
    [wikipedia page](https://en.wikipedia.org/wiki/Henrietta_Swan_Leavitt)
    or read some of
    [her papers](https://ui.adsabs.harvard.edu/#search/q=%20author%3A%22leavitt%2C%20henrietta%20swan).
    ''')

from .version import __version__
from .lightcurves import *
from .statistics import *
from .modeling import *
from .tools import *
from .fitting import *
from .tpf import *
from .imaging import *
from .photometry import *
from .mcmc import *
from .utilities import *



from ..imports import *
import skimage.io
from astropy.io import fits

def read_fits(path, ext_image=1):
    '''
    Read an image from a FITS file.
    '''

    # open the fits file
    hdu_list = fits.open(path)

    # make sure we're asking for a reasonable image extension
    ext_image = np.minimum(ext_image, len(hdu_list)-1)

    # extract the image data from the FITs file
    image = hdu_list[ext_image].data

    print('read a {} grayscale image from {}'.format(image.shape, path))

    # return the image
    return image

def read_rgb(path):
    '''
    Read an image (.jpg, .png, .tif, .gif)
    into three numpy arrays, one each for
    the brightness in Red, Green, Blue.

    Parameters
    ----------
    path : str
        The filename of the image to read.

    Returns
    -------
    r, g, b : arrays
        The brightness of the image in each of the
        red, green, blue wavelength ranges.
    '''

    # read a (rows x cols x 3) image
    rgb = skimage.io.imread(path, as_gray=False)

    if len(rgb.shape) == 2:
        # if grayscale already, set the red, green, blue channels to be equal
        r = rgb
        g = rgb
        b = rgb

    else:
        # pull out the red, green, blue channels
        r, g, b = rgb[:,:,0], rgb[:,:,1], rgb[:,:,2]

    # print an update
    print('read a {} RGB image from {}'.format(rgb.shape, path))

    # return the three separate images
    return r, g, b

def read_gray(path):
    '''
    Read an image (.jpg, .png, .tif, .gif)
    into one numpy array, where the individual
    RGB channels have been merged together
    into one single grayscale image.

    Parameters
    ----------
    path : str
        The filename of the image to read.

    Returns
    -------
    gray : array
        The brightness of the image, estimated
        from a weighted average across the red,
        green, and blue bandpasses.
    '''

    # load the image as gray-scale
    image = skimage.io.imread(path, as_gray=True)

    # print an update
    print('read a {} grayscale image from {}'.format(image.shape, path))

    # return the image
    return image

def read_image(path, **kw):
    '''
    Read an image, being flexible about what kind of image it is.

    Parameters
    ----------
    path : str
        The filename of the image to read.

    Returns
    -------
    gray : array
        The brightness of the image, as a 2D numpy array.
    '''

    if '.fit' in path:
        return read_fits(path, **kw)
    else:
        return read_gray(path, **kw)

def write_image(image, filename='image.jpg'):
    '''
    Write a numpy array to an image file.

    Parameters
    ----------
    image : array
        The image to save.
    filename : str
        The filename where the image should be saved.
    '''

    with warnings.catch_warnings():
        warnings.simplefilter('ignore')

        # save the image
        if '.fit' in filename:
            hdu = fits.PrimaryHDU(data=image).writeto(filename, overwrite=True)
        else:
            skimage.io.imsave(filename, image)

    # print an update
    print('saved {} image to {}'.format(image.shape, filename))

def compile_rgb(red, green, blue):
    incolor = np.zeros_like(red)[:,:,np.newaxis]*np.ones(3).astype(np.int)
    incolor[:,:,0] = red
    incolor[:,:,1] = green
    incolor[:,:,2] = blue
    return incolor

# read different image file types (png, tiff, giff)
# write a guesser to read an arbitrary image (based on filename)
# read a movies as a sequence of images (?)
# (add URL capability for the path)?



from ..imports import *
from ..imaging import visualize, io, simulations
from ..utilities import mkdir

directory = 'examples'
mkdir(directory)

def cartoon_rgb():
    x, y = np.meshgrid(np.linspace(0, 1, 1000), np.linspace(0, 1, 500))
    r = x
    g = y
    b = np.random.uniform(0, 1, r.shape)
    return r, g, b

def test_io():
    '''
    This tests the tools for reading and writing images.
    '''
    # create a test image
    xsize, ysize = 200, 100
    x = np.arange(xsize)
    image = np.random.uniform(0, 1.0, (ysize, xsize))
    rgb = io.compile_rgb(*cartoon_rgb())
    for suffix in ['jpg', 'png', 'gif']:
        filename = os.path.join(directory, 'image-gray.{}'.format(suffix))
        io.write_image(image, filename=filename)
        gray = io.read_gray(filename)
        r, g, b = io.read_rgb(filename)
        filename = os.path.join(directory, 'image-rgb.{}'.format(suffix))
        io.write_image(rgb, filename=filename)
        r, g, b = io.read_rgb(filename)

def test_rgb():
    r, g, b = cartoon_rgb()
    visualize.display_rgb(r, g, b)



from henrietta.photometry import *
import numpy as np
import matplotlib.pyplot as plt
import illumination as il

def test_photometry():
    ax = plt.subplot(111)
    im = create_test_array(N=1, xsize=100, ysize=100, nstars=10, single=False,seed=7)
    test = mean_centroid(im,[50,50],boxsize=60.,num_stars=1)
    print test
    return test



import numpy as np, matplotlib.pyplot as plt
from tqdm import tqdm

def create_test_array(N=5,
                      xsize=100,
                      ysize=100,
                      nstars=None,
                      single=False,
                      drift=0.0,
                      seed=None):
    '''
    Create a fake stack of images of stars
    (no cosmics injected, no jitter, Gaussian PDF).

    Parameters:
    -----------
    N : int
        The number of images.
    xsize : int
        The number of columns.
    ysize : int
        The number of rows.
    nstars : int
        The number of stars to include (defaults to be \propto to image area)
    single : bool
        Should this be just a single star in the center, or should it be many?
    drift : float
        The distance stars will drift per image, in pixels.

    Returns
    -------
    image_stack : array
        a simulated image of stars, with shape (N, ysize, xsize)
    '''

    # seed the random number generator (if None, will be truly random)
    np.random.seed(seed)

    # create images of x, y, and an empty one to fill with stars
    x1d = np.arange(0, xsize)
    y1d = np.arange(0, ysize)
    x, y = np.meshgrid(x1d, y1d)
    stars = np.zeros_like(x)

    # create N random position for stars
    if single:
        nstars = 1
        sx = np.atleast1d(np.random.normal(xsize / 2.0, 1))
        sy = np.atleast1d(np.random.normal(ysize / 2.0, 1))
    else:
        if nstars is None:
            nstars = np.minimum(int(xsize * ysize / 4), 1000)
        sx = np.random.uniform(0, xsize, nstars)
        sy = np.random.uniform(0, ysize, nstars)

    # set some background level
    bg = 30

    # create a semi-reasonable magnitude distribution for stars
    if single:
        topmag = 5
    else:
        topmag = 10

    # create fluxes for the star(s)
    sf = 10000 * 10**(-0.4 * np.random.triangular(0, topmag, topmag, nstars))

    # define the cartoon PSF for the stars
    sigma = 1.0

    # define a simple 2D Gaussian to represent the stars
    def gauss(x, y, x0, y0):
        return np.exp(-0.5 * ((x - x0)**2 + (y - y0)**2) / sigma**2)

    # either calculate one image at a time (with drift)
    if drift != 0.0:

        # create a stack of empty model images
        model = np.ones([N, ysize, xsize])

        # define some random direction in which stars will (linearly) drift
        theta = np.random.uniform(0, 2*np.pi)

        # loop over the images
        for i in tqdm(range(N)):
            # make up a nudge for this
            xnudge, ynudge = np.cos(theta) * i * drift, np.sin(theta) * i * drift

            # calculate a single fixed image of perfect (Gaussian) stars (at this nudge)
            stars = (sf * gauss(x[:, :, np.newaxis],
                                y[:, :, np.newaxis],
                                x0=sx + xnudge, y0=sy + ynudge)).sum(2)

            model[i, :, :] =  bg + stars
    # or calculate all the images at once
    else:
        # calculate a single fixed image of perfect (Gaussian) stars
        stars = (sf * gauss(x[:, :, np.newaxis],
                            y[:, :, np.newaxis],
                            x0=sx, y0=sy)).sum(2)

        # reshape that model into a cube
        model = (bg + stars)[np.newaxis, :, :] * np.ones(N).reshape((N, 1, 1))

    # add some Poisson noise to the cube of image
    image = np.random.normal(model, np.sqrt(model))

    return image



try:
    from .simulations import *
except ImportError:
    print("""
    The image input/output tools were not loaded, probabaly because
    astroquery is unavailable. Try running `pip install astroquery` to
    install it.
    """)

try:
    from .visualize import *
except ImportError:
    print("""
    The image input/output tools were not loaded, probabaly because
    illumination is unavailable. Try running `pip install illumination`
    to install it.
    """)

try:
    from .io import *
except ImportError:
    print("""
    The image input/output tools were not loaded, probabaly because
    skimage is unavailable. Try running `pip install skimage` to
    install it.
    """)



from henrietta.photometry import *
import numpy as np
import matplotlib.pyplot as plt
import illumination as il

def test_photometry():
    ax = plt.subplot(111)
    im = create_test_array(N=1, xsize=100, ysize=100, nstars=10, single=False,seed=7)
    test = photometry(ax,im[0],[[54,91],[44,38]],ap_size=3,r_in=8,r_out=12,back_photo=True)
    return test

if __name__ == '__main__':
    test_photometry()



import numpy as np
import matplotlib.pyplot as plt
from .goodnesses import *
from .tools import *

try:
    import batman
except:
    print("""
    `batman-package` is not installed on this computer.

    You can use most of the henrietta tools with no problem,
    but you will not be able to generate transit models :-(

    You can try installing it with the command
        `pip install batman-package`
    """)
import lightkurve
from lightkurve import LightCurve
from .statistics import *

def BATMAN(t,
           period = 1.0, #days
           t0 = 0, #time of inferior conjunction
           radius = 0.1, #Rp/R*
           a = 10.0, #semi-major axis in a/R*
           b = 0.0, #impact parameter in stellar radii
           baseline = 1.0, #units are whatever your flux units come in
           ld1 = 0.1,#using a basic limb darkening
           ld2 = 0.3,
           ):
    '''
    This function returns a model transit light curve
    for a given array of times and set of parameters.
    It currently assumes a circular orbit.

    Parameters
    ----------
    t : array
        An array of times, in units of days.

    period : float
        The orbital period of the planet, in units of days.

    t0 : float
        One mid-transit time, in units of days.

    radius : float
        The radius of the planet, in units of stellar radii.
        (This is sometimes also called Rp/R*).

    a : float
        The orbital distance (semimajor axis) of the planet, in stellar radii.
        (This is sometimes also called a/R*).

    b : float
        The transit impact parameter of the planet, in stellar radii.
        (This is closely related to the inclination.)

    baseline : float
        The out-of-transit flux level, in whatever units you want.

    ld : list or array of floats
        The limb-darkening coefficients, for a quadratic limb-darkening.
    '''

    # create a batman transit parameter object
    params = batman.TransitParams()

    params.t0 = t0                      #time of inferior conjunction ()
    params.per = period             #period in days
    params.rp = radius         #planet radius (in units of stellar radii)
    params.a = a                      #semi-major axis (in units of stellar radii)
    params.inc = np.arccos(b/a)*180/np.pi                   #orbital inclination (in degrees)
    params.ecc = 0.                      #eccentricity
    params.w = 90.                       #longitude of periastron (in degrees)
    params.u = [ld1, ld2]        #limb darkening coefficients [u1, u2]
    params.limb_dark = "quadratic"       #limb darkening model

    # initialize a batman model for the given times
    m = batman.TransitModel(params, t)

    # calculate a light curve (at the pre-set times)
    flux = m.light_curve(params)*baseline

    # return those light curve values
    return flux

def example_transit_model( period = 0.5, #days
                           t0 = 0, #time of inferior conjunction
                           radius = 0.1, #Rp/R*
                           a = 10.0, #semi-major axis in a/R*
                           b = 0.0, #impact parameter in stellar radii,
                           tmin=-0.5, tmax=0.5, cadence=1.0/60.0/24.0,
                           ylim=[0.985, 1.005],
                           ax=None,
                           **plotkw):
    '''
    This function makes an example plot of a model transit light curve.

    Parameters
    ----------

    period : float
        The orbital period of the planet, in units of days.

    t0 : float
        One mid-transit time, in units of days.

    radius : float
        The radius of the planet, in units of stellar radii.
        (This is sometimes also called Rp/R*).

    a : float
        The orbital distance (semimajor axis) of the planet, in stellar radii.
        (This is sometimes also called a/R*).

    b : float
        The transit impact parameter of the planet, in stellar radii.
        (This is closely related to the inclination.)

    tmin : float
        The minimum time to plot, in days.

    tmax : float
        The maximum time to plot, in days.

    cadence : float
        The cadence (time-step) to use, in days.

    ylim : list
        The ylimits for the plot.

    ax : matplotlib.axes object
        If an ax is fed in as a keyword argument, the plot
        will be added to that existing plot axes. To include
        multiple different models on the same plot, say:

        ax = example_transit_model()
        ax = example_transit_model(rp=0.042, ax=ax)
        ax = example_transit_model(rp=0.042, b=0.9, ax=ax)
        ...

    plotkw : dict
        Any extra keywords will be passed along to `matplotlib.pyplot.plot`

    Returns
    -------

    plotted_ax : matplotlib.axes object
        The axes into which the model was was plotted.
    '''

    # create an array of times and flux values
    t = np.arange(tmin, tmax, cadence)
    f = BATMAN(t=t, period=period, t0=t0, radius=radius, a=a, b=b)

    # make a lightkurve LightCurve object
    # model_lc = LightCurve(time=t, flux=f)

    # figure out with ax to plot in
    if ax is None:
        ax = plt.gca()
    else:
        plt.sca(ax)

    # update the label, if it doesn't already exist
    if 'label' != plotkw:
        plotkw['label'] = 'period={period},t0={t0},radius={radius},a={a},b={b}'.format(**locals())

    # make the plot, set the y limits
    plt.plot(t, f, **plotkw)
    plt.ylim(*ylim)

    # set some ylimits
    plt.xlabel('Time (days)')
    plt.ylabel('Relative Flux')

    # return the current axes, in case someone wants to plot into them again
    return ax

def simulate_transit_data(N=1e6, cadence=2.0/60.0/24.0, duration=3.0, tmin=0.0,  **kw):
    '''
    This function will generate a simulated LightCurve dataset
    with a given fractional noise (sigma) and time spacing (cadence),
    with a transit injected into it (whose parameters are set by any
    extra keyword arguments that you feed in, like `period`, `radius`, ...)

    Parameters
    ----------

    N : float
        The average number of photons expected per exposure, to
        set the standard deviation of the noise.

    cadence : float
        The integration time of the measurements, in days.

    duration : float
        The total length of time covered by the light curve.

    **kw : dict
        Any additional keywords will be passed onward to the
        batman model to set the parameters of the transit model.
        Valid additional keywords are period, t0, radius, a, b, ld.

    Returns
    -------
    lc : LightCurve
        A simulated lightkurve LightCurve, with a transit injected,
        and the specified noise.
    '''
    noise = create_photon_lightcurve(N=N, cadence=cadence, duration=duration).normalize()
    noise.time += tmin
    flux = BATMAN(noise.time, **kw)
    return LightCurve(time=noise.time, flux=flux*noise.flux, flux_err=noise.flux_err, time_format='jd')

def plot_with_transit_model(lc,
                           period = 1.0,
                           t0 = 0,
                           radius = 0.1,
                           a = 10.0,
                           b = 0.0,
                           baseline = 1.0,
                           ld1 = 0.1, ld2 = 0.3,
                           planet_name='',
                           goodness=None,
                           show_errors=False,
                           datakw={},
                           modelkw={}):
    '''
    This function will take in a lightcurve for a planet
    with a given set of transit parameters (period, t0, radius, a, b, baseline)
    and will plot a batman light curve model. Limb-darkening coefficients
    can also be specified in the form [u1,u2].

    Parameters
    ----------
    lc : LightCurve object
        Data contained in a lightkurve LightCurve object.

    period : float
        The orbital period of the planet, in units of days.

    t0 : float
        One mid-transit time, in units of days.

    radius : float
        The radius of the planet, in units of stellar radii.
        (This is sometimes also called Rp/R*).

    a : float
        The orbital distance (semimajor axis) of the planet, in stellar radii.
        (This is sometimes also called a/R*).

    b : float
        The transit impact parameter of the planet, in stellar radii.
        (This is closely related to the inclination.)

    baseline : float
        The out-of-transit flux level, in whatever units you want.

    ld : list or array of floats
        The limb-darkening coefficients, for a quadratic limb-darkening.

    planet_name : string
        The name of the planet, which will be displayed as the title

    goodness : function
        A function that takes an array of values for (data-model),
        and returns a goodness of fit metric.

    datakw : dictionary
        A dictionary with keywords that will be passed to the plt.plot()
        command that displays the actual data points.

    modelkw : dictionary
        A dictionary with keywords that will be passed to the plt.plot()
        command that displays the actual the smooth model.
    '''

    # create a high-resolution grid of times to plot
    highres_time = np.arange(lc.time[0],lc.time[-1],1.0/60.0/24.0)

    # figure out the right time format
    if isinstance(lc, lightkurve.lightcurve.FoldedLightCurve):
        epoch = 0.0
        period = 1.0
    else:
        epoch = find_appropriate_epoch(lc, t0)

    # craete a model of the flux at the light curve times
    model_flux = BATMAN(baseline = baseline,
                radius = radius,
                period = period,
                a = a,
                b = b,
                t0 = epoch,
                ld1 = ld1, ld2 = ld2,
                t = lc.time)

    # create a model of the flux at high resolution
    model_plot = BATMAN(baseline = baseline,
                radius = radius,
                period = period,
                a = a,
                b = b,
                t0 = epoch,
                ld1 = ld1, ld2 = ld2,
                t = highres_time)

    # calculation the difference between the data and the model
    residual = (lc.flux - model_flux)


    f, (a0, a1) = plt.subplots(2,1, gridspec_kw = {'height_ratios':[4,1]},figsize=(10,5),sharex=True)


    a0.set_ylabel('Flux')#,fontsize=18)
    actualdatakw = dict( alpha=0.5, color='royalblue',markersize='5', markeredgecolor='none', zorder=0)
    actualdatakw.update(**datakw)

    actualmodelkw = dict(zorder=1,color='k')
    actualmodelkw.update(**modelkw)

    summary = 'BATMAN(period={period},t0={t0},radius={radius},a={a},b={b})'.format(**locals())
    a0.plot(highres_time,model_plot,label='Model', **actualmodelkw)

    if show_errors:
        a0.errorbar(lc.time,lc.flux,yerr=lc.flux_err,fmt='o',label='Data', **actualdatakw)
    else:
        a0.plot(lc.time,lc.flux,label='Data', marker='o', linewidth=0, **actualdatakw)


    a0.legend()#loc='upper left', bbox_to_anchor=(1,1))

    if show_errors:
        a1.errorbar(lc.time,residual,yerr=lc.flux_err, **actualdatakw)
    else:
        a1.plot(lc.time,residual,marker='o',linewidth=0, **actualdatakw)
    a1.axhline(0, **actualmodelkw)
    a1.set_ylim(0-1.5*np.nanmax(np.abs(residual)),0+1.5*np.nanmax(np.abs(residual)))
    a1.set_ylabel('Residuals')
    plt.xlabel('Time (days)')

    if planet_name is '':
        title = ''
    else:
        title = '{} | '.format(planet_name)

    if goodness is None:
        title += summary
        gof = None
    else:
        plt.show()
        gof = goodness(residual/lc.flux_err)
        title += '{} | {}={:.4}'.format(summary, goodness.__name__, gof)
    a0.set_title(title, fontsize=10)

    return gof



import numpy as np

def guessncheck(objfunc, model, x, y, N=100):
    '''
    This function guesses and checks lots of different models,
    and returns the best oneself.

    Parameters
    ----------
    objfunc : function
        The function that tells us how good a fit is, based on the model
        and the data (this is fed automitically by our "fitter").

    model : astropy.model
        The astropy model we're trying to fit.

    x : numpy.ndarray
        The independent values (x).

    y : numpy.ndarray
        The dependent values (y).

    N : int
        How many models should we try?
    '''

    # assume our first model has the best parameters (initially)
    best = model.parameters

    # create a tester model to play with
    tester = model.copy()

    # create an empty array for our goodness of fit
    gof = np.ones(N)*np.inf

    # do a bunch of iterations
    for i in range(N):

        # set the parameters, by drawing randomly from their bounds
        for w, p in enumerate(model.param_names):

            # only randomize bounded and un-fixed variables
            if None in model.bounds[p] or model.fixed[p]:
                pass
            else:
                tester.parameters[w] = np.random.uniform(*model.bounds[p])

        # calculate the goodness of fit
        gof[i] = objfunc(measured_vals=y,
                         updated_model=tester,
                         weights=None,
                         x=x)

        # store the best set of parameters
        if gof[i] <= np.nanmin(gof):
            best = tester.parameters

    return best



from lightkurve import KeplerLightCurveFile, lightcurve, search_lightcurvefile
import matplotlib.pyplot as plt
import numpy as np
from .tools import *
from lightkurve.lightcurve import LightCurve
from lightkurve.collections import LightCurveFileCollection


def download_kepler_lc(star='Kepler-186',
                       quality_bitmask='hard',
                       kind='PDCSAP_FLUX', **kw):
    '''
    This function is a wrapper to download one or more quarters of Kepler
    lightcurve data, and extract a LightCurve object from it.

    When downloading a Kepler LightCurveFile from online, we get lots
    of information, including multiple different kinds of light curves
    for the same star. The goal of this little wrapper is to make it
    easy to get just one single light curve to play with.

    This has been tested only on data from the main Kepler mission.
    For K2 data, you may need to download a TPF.

    Parameters
    ----------

    star: str, int
        The name of the star whose light curve you want to download.
        This can be a name ("Kepler-42", "KOI-961") or a KIC number
        (8561063)

    quarter: str, int
        Which Kepler quarter to download?
            'all' = download the entire mission of data
            1, 2, 3 ... = download a specific quarter

    cadence: str
        'long' for Kepler long cadence
        'short' for Kepler short cadence

    quality_bitmask: str, int
        'none' = ignore no data
        'default' = ignore data with serious quality issues
        'hard' = be a little more cautious in what we call good data
        'hardest' = be too cautious about what we call good data

    kind: str
        "SAP_FLUX" = simple aperture photometry
        "PDCSAP_FLUX" = SAP, with pre-search data conditioning applied to get
        rid of some instrumental systematics from the light curves (but watch
        out! some astrophysical signals might be messed up too!)

    kw : dict
        Additional keywords will be passed to `.from_archive`.

    Returns
    -------

    lc: LightCurve object
        This is a `lightkurve`-style LightCurve object, which contains
        the attributes `lc.time` (times in JD) and `lc.flux` (the brightness
        of the star), as well as lots of methods for analysis and plotting.

    '''

    # download a KeplerLightCurveFile (or list of them) from the MAST archive
    lcf = search_lightcurvefile(star,**kw).download_all(quality_bitmask=quality_bitmask)

    # if a list, stitch things together (crudely! will be terrible for SAP!)
    if isinstance(lcf, LightCurveFileCollection):

        # make a normalized light curve from the first light curve file
        lc = lcf[0].get_lightcurve(kind).normalize()

        # append the normalized light curves from all other light curve files
        for f in lcf[1:]:
            thisquarter = f.get_lightcurve(kind)
            lc = lc.append(thisquarter.normalize())

    # if a single quarter, simply return that light curve
    else:
        lc = lcf.get_lightcurve(kind)

    # return the light curve
    return lc

def download_lc(*args, **kwargs):
    return download_kepler_lc(*args, **kwargs)


def locate_transits(lc, period, t0=0, name=None, color='green', **kw):

    '''

    This function takes in a lightcurve object along with the planet's
    period and epoch in order to identify on the plot each expected location
    of a transit

    lc should be a lightkurve object which has times and fluxes.
    period should be in days
    t0 should be in BJD
    '''

    # pull out time and flux arrays
    time = lc.time
    flux = lc.flux

    # figure out the right time format
    if isinstance(lc, lightcurve.FoldedLightCurve):
        epoch = 0.0
        period = 1.0
    else:
        if lc.time_format == 'bkjd':
            epoch = bjd2bkjd(t0)
        elif lc.time_format == 'btjd':
            epoch = bjd2btjd(t0)
        else:
            epoch = t0

    # calculate the integer orbit numbers, and mid-transit locations
    allorbitnumbers = np.round((lc.time - epoch)/period).astype(np.int)
    n = np.arange(np.min(allorbitnumbers), np.max(allorbitnumbers))
    transit_loc = n*period + epoch

    #avg_flux = np.median(flux)
    #fig,ax = plt.subplots()
    #plt.title('Transit locations')
    #plt.xlabel('Time (JD)')
    #lt.ylabel('Flux')
    #plt.xlim(time[0],time[-1])
    #plt.scatter(time,flux)

    for i, this_transit in enumerate(transit_loc):
        if i == 0:
            label=name or '{period}{multiply}n + {epoch:.5f}'.format(multiply=r'$\times$', **locals())
        else:
            label=None
        plt.axvline(this_transit, ymin=0.9, ymax=0.95, alpha=0.4, color=color, label=label, **kw)
    return transit_loc


def extract_transits(lc, period, epoch, window=0.05):
    '''
    This function splits a light curve into two complementary
    light curves: one that includes just the transits (and a
    bit of a window surrounding them) and one that contains all
    the other non-transit parts of the light curve.

    Parameters
    ----------

    lc: object
        The 'lightkurve' object which contains transit data (time, flux) that
        we are analyzing.

    period: float
        The period (days) of the planet's orbit.

    epoch: float
        The JD value reported for the mid-transit time of this planet. Epoch
        is used to find other transits in the data.

    window: float
        The transit duration (in days), used here to specify a window of data
        points to extract as the in-transit data.

    Returns
    -------

    transits: LightCurve object
        This is a `lightkurve`-style LightCurve object, which contains
        the attributes `lc.time` (times in JD) and `lc.flux` (the brightness
        of the star). This object contains the data points between ingress
        egress of a transit.

    notransits: LightCurve object
        This is a `lightkurve`-style LightCurve object, which contains
        the attributes `lc.time` (times in JD) and `lc.flux` (the brightness
        of the star). This object contains the data points outside of a transit.

    '''

    time = lc.time
    flux = lc.flux
    error = lc.flux_err

    n = np.round((time-epoch)/period)
    n_transit = np.unique(n)

    mid_transit_times = (period*n_transit + epoch)

    ingress = []
    egress = []

    for i in range(len(n_transit)):
        if (mid_transit_times[i] >= lc.time[0]):
            if (mid_transit_times[i] <= lc.time[-1]):

                t0 = mid_transit_times[i]
                ingress.append(t0 - window/2.0)
                egress.append(t0 + window/2.0)

    transit_indices = []

    for i in range(len(time)):
        for j in range(len(ingress)):
            if time[i] >= ingress[j]:
                if time[i] <= egress[j]:
                    transit_indices.append(i)

    transit_time = time[transit_indices]
    transit_flux = flux[transit_indices]
    transit_error = error[transit_indices]

    oot_time = np.delete(time,transit_indices)
    oot_flux = np.delete(flux,transit_indices)
    oot_error = np.delete(error,transit_indices)

    transits = LightCurve(transit_time, transit_flux, transit_error)
    notransits = LightCurve(oot_time, oot_flux, oot_error)

    return transits, notransits



import thefriendlystars as tfs
import astropy.units as u
from ..imports import *

def simulate_optics(target='Kepler-42', collectingarea=1.0, pixelscale=21.0, fov=3*60, background=0.0, **kw):
    '''

    Parameters
    ----------
    collectingarea : float
        The collecting area of the telescope, in units square meters.

    pixelscale : float
        The pixel scale, in units of arcseconds/pixel.

    fov : float
        The field-of-view of the telescope, in arcseconds.

    background : float
        What is the diffuse sky brightness,
        in units of photons/s/arcsecond**2?

    Returns
    -------
    flux_image : np.ndarray
        An image of the flux that would hit a detector,
        in units of photons/s/pixel. It is a *perfect*
        image, in the sense that no noise has been
        added to it.
    '''

    center = tfs.parse_center(target)

    catalog = tfs.Gaia.from_cone(center, radius=fov*u.arcsec/np.sqrt(2))
    # figure out the relative positions of the stars
    dra = ((catalog.ra - center.ra)*np.cos(center.dec)).to('arcsec').value
    ddec = (catalog.dec - center.dec).to('arcsec').value
    sx, sy = dra/pixelscale, ddec/pixelscale

    magnitudes = catalog.magnitude

    # (very approximate!)
    # 0 mag = 10^6 photons/s/cm^2
    # 0 mag = 10^10 photons/s/m^2
    zero = 1.0e6*(100)**2
    photonfluxes = zero*10**(-0.4*magnitudes)

    xsize = int(np.ceil(fov/pixelscale/2.0))
    ysize = xsize

    # create 2D arrays
    x1d = np.arange(-xsize, xsize)
    y1d = np.arange(-ysize, ysize)
    x, y = np.meshgrid(x1d, y1d)
    stars = np.zeros_like(x)

    # define the cartoon PSF for the stars
    sigma = 1.0

    def gauss(x, y, x0, y0):
        return 1.0/2*np.pi/sigma**2*np.exp(-0.5 * ((x - x0)**2 + (y - y0)**2) / sigma**2)

    # create a cube with an image for each star, and sum them together into one image
    stars = (photonfluxes * gauss(x[:, :, np.newaxis],
                                  y[:, :, np.newaxis],
                                  x0=sx, y0=sy)).sum(2)

    # create a model images
    model = (background + stars)#[np.newaxis, :, :] * np.ones(N).reshape((N, 1, 1))

    if type(model) == np.ma.core.MaskedArray:
        model = model.data
    return model*collectingarea

def simulate_detector(image, exptime=1.0, quantumefficiency=1.0, readnoise=10.0, **kw):
    '''
    Create a simulate image indicating the total number of
    photons that a detector will record per pixel, in a
    given exposure time.

    Parameters
    ----------
    image : np.ndarray
        An example flux image, in units of photons/s.
        For example, the output of `simulate_optics`
        would be a great input here!

    exptime : float
        The total exposure time, in units of seconds.
        How long do we let the detector collect photons
        before reading out the detector.

    quantumefficiency : float
        What fraction of incoming photons actually gets
        recorded as photons?
            1.0 is a perfect detector.
            0.0 is no detector at all.

    readnoise : float
        How much noise does the detector add to each pixel
        when it is read out of the detector? This is effectively
        in units of photons/pixel/exposure.

    Returns
    -------
    photon_image : np.ndarray
        An image of the total number of photons that
        would be recorded by the detector, in units of
        photons/pixel. It is a *noisy* image, in the sense
        it is a single realization of the image, and it
        will change if you run the function again.
    '''

    # calculate the expected total number of photons/pixel
    expectation = image*exptime*quantumefficiency

    # calculate a noise image, assuming Gaussian statistics
    noise = np.sqrt(expectation + readnoise**2)

    return np.random.normal(expectation, noise)

def simulate_image(*args, **kwargs):
    '''
    Make an end-to-end simulation of an astronomical image,
    combining the steps of `simulate_optics` with `simulate_detector`.

    Parameters
    ----------
        (anything that can be fed into `simulate_optics` or `simulate_detector`)

    Returns
    -------
    sim : array
        A simulated image.
    '''
    optics = simulate_optics(*args, **kwargs)
    exposure = simulate_detector(optics, **kwargs)
    return exposure



'''
This module contains functions and class definitions to extend the
astropy.modeling toolkit to be visualizeable.
'''

from __future__ import print_function
from .imports import *
from .optimizers import *
from .modeling import BATMAN
from .utilities import decide_writer
from .goodnesses import *

from astropy.modeling import models, fitting, optimizers, statistic, custom_model


# define a custom model, based off our BATMAN function
BatmanTransit = custom_model(BATMAN)

@custom_model
def TrapezoidTransit(t, delta=0.01, P=1, t0=0, T=0.1, tau=0.01, baseline=1.0):

    """
    One dimensional Trapezoid Transit model,
    using the symbols defined for a circular
    transit approximation in Winn (2010).

    This is a fittable astropy model.

    Parameters
    ----------
    delta : float
        The depth of the transit, as a fraction of the out-of-transit flux.
    P : float
        The perio of the planet, in days.
    t0 : float
        Mid-transit time of the transit, in days.
    T : float
        The duration of the transit (from mid-ingress to mid-egress), in days.
    tau : float
        The duration of ingress/egress, in days.

    See Also
    --------
    astropy.modeling.models.Trapezoid1D

    Examples
    --------
    .. plot::
        :include-source:

        import numpy as np
        import matplotlib.pyplot as plt

        from henrietta.fitting import TrapezoidTransit

        plt.figure()
        model = TrapezoidTransit()
        t = np.arange(-5, 5, .01)

        for x in np.linspace(0.02, 0.2, 4):
            model.delta = x
            model.T = x
            plt.plot(t, model(t), lw=2)

        plt.show()
    """

    # calculate a phase-folded time (still in units of days)
    x = (t-t0 + 0.5*P) % P - 0.5*P

    # Compute the four points where the trapezoid changes slope
    # x1 <= x2 <= x3 <= x4

    if tau > T:
        x1 = -tau
        x2 = 0
        x3 = 0
        x4 = tau
    else:
        x1 = - (T+tau)/2.0
        x2 = - (T-tau)/2.0
        x3 = (T-tau)/2.0
        x4 = (T+tau)/2.0

    # Compute model values in pieces between the change points
    range_a = np.logical_and(x >= x1, x < x2)
    range_b = np.logical_and(x >= x2, x < x3)
    range_c = np.logical_and(x >= x3, x < x4)

    if tau == 0:
        slope = np.inf
    else:
        slope = delta/tau
    val_a = 1 - slope * (x - x1)
    val_b = 1 - delta
    val_c = 1 - slope * (x4 - x)
    result = np.select([range_a, range_b, range_c], [val_a, val_b, val_c], default=1)*baseline
    return result

def setup_transit_model(period=1.58,
                        t0=0.0,
                        radius=[0.0, 0.5],
                        a=[3.0, 50.0],
                        b=0.385,
                        baseline=1.0,
                        ld1=0.1, ld2=0.3):
    '''
    This function sets up an astropy transit model, which can then be used for fitting.

    Parameters that are fed in as single values will be held fixed.

    Parameters that are fed in as two-element lists will be varied,
    using the two values as their allowable bounds.

    Parameters
    ----------
    (same as we've seen in other transit modeling)
    '''
    inputs = {}
    names = ['period', 't0', 'radius', 'a', 'b', 'baseline', 'ld1', 'ld2']

    # set up the initial values
    for k in names:
        inputs[k] = np.mean(locals()[k])

    # decide which parameters are fixed and which are not
    model = BatmanTransit(**inputs)
    for k in names:
        if len(np.atleast_1d(locals()[k])) == 2:
            model.fixed[k] = False
            model.bounds[k] = locals()[k]
        else:
            model.fixed[k] = True

    return model

def setup_line_model(slope=[0, 5], intercept=[-10, 10]):
    '''
    This function sets up an astropy line model, which can then be used for fitting.

    Parameters that are fed in as single values will be held fixed.

    Parameters that are fed in as two-element lists will be varied,
    using the two values as their allowable bounds.

    Parameters
    ----------
    slope : float, or 2-element list
        The
    
    '''
    inputs = {}
    names = ['slope', 'intercept']

    # set up the initial values
    for k in names:
        inputs[k] = np.mean(locals()[k])

    # decide which parameters are fixed and which are not
    model = models.Linear1D(**inputs)
    for k in names:
        if len(np.atleast_1d(locals()[k])) == 2:
            model.fixed[k] = False
            model.bounds[k] = locals()[k]
        else:
            model.fixed[k] = True

    return model


class VisualizedStatistic:
    '''
    Objects that inherit from VisualizedStatistic should be a drop-in
    replacement for the astropy.modeling.statistic.leastsq. The simplest
    class definiton that modifies this would simply be to redefine the
    `.name` attribute and the `.goodness()` method.
    '''

    # this may appear in labels
    #name = 'sumofsquares'

    def __init__(self, goodness=sumofsquares, fitter=None):
        '''
        Initialize a visualized statistic, which is connected to a fitter.
        '''
        self.goodness = goodness
        self.name = goodness.__name__
        self.fitter = fitter

    def __call__(self, measured_vals, updated_model, weights, x, y=None):
        """
        Generic goodness-of-fit statistic.

        Parameters
        ----------
        measured_vals : `~numpy.ndarray`
            Measured data values.
        updated_model : `~astropy.modeling.Model`
            Model with parameters set by the current iteration of the optimizer.
        weights : `~numpy.ndarray`
            Array of weights to apply to each residual.
        x : `~numpy.ndarray`
            Independent variable "x" to evaluate the model on.
        y : `~numpy.ndarray`, optional
            Independent variable "y" to evaluate the model on, for 2D models.

        Returns
        -------
        res : float
            The sum of least squares.
        """


        if y is None:
            model_vals = updated_model(x)
        else:
            model_vals = updated_model(x, y)
        if weights is None:
            residuals = (model_vals - measured_vals)
        else:
            residuals = (weights * (model_vals - measured_vals))

        gof = self.goodness(residuals)

        #print(self.fitter._iterations, gof)

        # plot the updated model on the data
        plt.sca(self.fitter.ax['data'])

        # make the previous models fade more, then add a new model
        for l in self.fitter.plotted['data']['models']:
            l.set_alpha(0.95*l.get_alpha())
        newline = plt.plot(x, updated_model(x), zorder=-1, color=l.get_color(), alpha=l.get_alpha(), linewidth=l.get_linewidth())[0]
        self.fitter.plotted['data']['models'].append(newline)

        if len(self.fitter._gof) == 0:
            best = True
        else:
            best = gof < np.min(self.fitter._gof)

        if best:

            # update the best model
            self.fitter.plotted['data']['best'].set_data(x, updated_model(x))

            # update the title
            plt.suptitle('Best Model ({} = {:.4})\n{}'.format(self.name, gof, repr(updated_model)))

            # update the goodness plot
            self.fitter.plotted['gof']['best'].set_data(self.fitter._iterations, gof)



        # plot the current step number and goodness of fit
        plt.sca(self.fitter.ax['gof'])
        self.fitter._iterations += 1
        self.fitter._gof.append(gof)
        self.fitter.plotted['gof']['all'].set_data(np.arange(self.fitter._iterations), self.fitter._gof)

        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            self.fitter.ax['gof'].set_xlim(0, self.fitter._iterations+1)
            self.fitter.ax['gof'].set_ylim(np.min(self.fitter._gof), np.max(self.fitter._gof))

        # update the parameters (kludge?)
        interesting = list(self.fitter.plotted['params'].keys())
        for i, p in enumerate(updated_model.param_names):
            if p in interesting:
                self.fitter._parameters[p].append(updated_model.parameters[i])
        # update the goodness
        self.fitter._parameters[self.name] = self.fitter._gof

        # update the plots
        for i, pi in enumerate(interesting):
            for j, pj in enumerate(interesting):
                x = self.fitter._parameters[pj]
                y = self.fitter._parameters[pi]
                if i>j:
                    #print(pj, x)
                    #print(pi, y)
                    #print('---')
                    self.fitter.plotted['params'][pi][pj]['all'].set_data((x, y))
                    if best:
                        self.fitter.plotted['params'][pi][pj]['best'].set_data((x[-1], y[-1]))

                    if pi == self.name:
                        with warnings.catch_warnings():
                            warnings.simplefilter("ignore")
                            self.fitter.ax['params'][pi][pj].set_ylim(np.nanmin(y), np.nanmax(y))



        if self.fitter.animate:
            self.fitter.writer.grab_frame()
            print('Saving frame {} to {}.'.format(self.fitter._iterations, self.fitter.writer.outfile), end='\r')
        else:
            print('Calculating test model #{}.'.format(self.fitter._iterations), end='\r')
        return self.goodness(residuals)

class VisualizedFitter(fitting.Fitter):
    '''
    Objects that inherit from a VisualizedFitter can do the normal
    stuff an astropy.fitting.Fitter can do, but they also visualize
    the process with panels that show what's happening.
    '''
    name = 'fitter'

    """
    def __init__(self, optimizer=optimizers.Simplex, goodness=sumofsquares, **kwargs):

        # shortcuts to make it easier to keep track of the optimizer and statistic
        self.optimizer = optimizer
        self.statistic = VisualizedStatistic(fitter=self, goodness=goodness)
        super().__init__(optimizer=self.optimizer, statistic=self.statistic)
    """

    def visualize(self, model, x, y, filename=None, animate=True, **kwargs):
        '''
        Yo
        '''

        self.animate = animate
        label = '{}-{}'.format(self.name, self.statistic.name)
        if self.animate:
            # make sure we have a filename set
            if filename is None:
                filename = '{}.mp4'.format(label)
            # figure out which animation writer to use
            self.writer = decide_writer(filename, fps=10)
            dpi = 200



        # restart our counter and goodness of fit score
        self._iterations = 0
        self._gof = []

        # set up a figure and some axes
        self.fi = plt.figure(figsize=(10, 7))
        gs = plt.matplotlib.gridspec.GridSpec(2, 2, height_ratios=[3, 1], hspace=0.25, wspace=0.25)

        # create some axes for our plot, and populate them with empty plots
        self.ax, self.plotted = {}, {}

        # create a space to plot the data vs. the model
        self.ax['data'] = plt.subplot(gs[0,0])

        allcolor = 'gray'
        bestcolor='mediumseagreen'
        self.plotted['data'] = dict(data=plt.plot(x, y, '.k', zorder=0),
                                    models=[plt.plot(x, model(x), zorder=-1, color=allcolor, alpha=1, linewidth=1)[0]],
                                    best=plt.plot(x, model(x), color=bestcolor, linewidth=3)[0])
        # (fuss with the axes)
        span = np.nanmax(y) - np.nanmin(y)
        self.ax['data'].set_ylim(np.nanmin(y)-span/2, np.nanmax(y)+span/2)
        self.ax['data'].set_xlabel('x')
        self.ax['data'].set_ylabel('y')


        # create a space to plot the goodness-of-fit vs time
        self.ax['gof'] = plt.subplot(gs[1,:])

        allkw = dict(marker='o', markersize=5, markeredgecolor='none', color=allcolor)
        bestkw = dict(marker='o', markersize=15, markeredgecolor='none', alpha=0.5, color=bestcolor)
        self.plotted['gof'] = dict(all=plt.plot([],[], **allkw)[0],
                                   best=plt.plot([],[], **bestkw)[0])
        # (fuss with the axes)
        self.ax['gof'].set_xlabel('# of Model Calculations')
        self.ax['gof'].set_ylabel('Goodness-of-Fit\n({})'.format(self._stat_method.name))

        # create a space for plotting the parameters
        interesting = [p for p in model.param_names if model.fixed[p] == False]
        # (include the goodness of fit values)
        interesting += [self._stat_method.name]
        N = len(interesting)-1
        # (grid a triangle of axes)
        gs_param = plt.matplotlib.gridspec.GridSpecFromSubplotSpec(N, N, gs[0,1])
        self.ax['params'] = {p:{} for p in interesting}
        self.plotted['params'] = {p:{} for p in interesting}

        # (decide the x/y limits for each box)
        limits = dict(**model.bounds)
        limits[self._stat_method.name] = (None, None)

        # loop through the triangle of parameter pairs
        for i, pi in enumerate(interesting):
            for j, pj in enumerate(interesting):
                if i>j:
                    # (create ax at this row/column)
                    ax = plt.subplot(gs_param[i-1,j])
                    self.ax['params'][pi][pj] = ax
                    with warnings.catch_warnings():
                        warnings.simplefilter("ignore")
                        ax.set_xlim(*limits[pj])
                        ax.set_ylim(*limits[pi])

                    #print(limits[pj], limits[pi])
                    if j == 0:
                        ax.set_ylabel(pi)
                    else:
                        plt.setp(ax.get_yticklabels(), visible=False)
                    if i == N:
                        ax.set_xlabel(pj)
                    else:
                        plt.setp(ax.get_xticklabels(), visible=False)

                    self.plotted['params'][pi][pj] = dict(all=plt.plot([],[], '.', **allkw)[0],
                                                          best=plt.plot([],[], '.', **bestkw)[0])

        # create a dictionary to store the parameter arrays in
        self._parameters = {p:[] for p in interesting}

        if self.animate:
            with self.writer.saving(self.fi, filename, dpi or figure.get_dpi()):
                fitted = self.__call__(model, x=x, y=y, **kwargs)
        else:
            fitted = self.__call__(model, x=x, y=y, **kwargs)
        return fitted


class VisualizedSimplexFitter(fitting.SimplexLSQFitter, VisualizedFitter):
    name = 'simplex'
    def __init__(self, goodness=sumofsquares, **kwargs):

        super().__init__(**kwargs)

        # shortcut to keep track of the optimizer and statistic
        self.optimizer = optimizers.Simplex
        self.statistic = VisualizedStatistic(goodness=goodness, fitter=self)

        # tell the optimizer which statistic to try to minimize
        self._stat_method = self.statistic


class VisualizedGuessNCheckFitter(VisualizedFitter):
    name = 'guessncheck'
    def __init__(self,  goodness=sumofsquares, **kwargs):

        # shortcut to keep track of the optimizer and statistic
        self.optimizer = guessncheck
        self.statistic = VisualizedStatistic(goodness=goodness, fitter=self)
        super().__init__(optimizer=self.optimizer, statistic=self.statistic, **kwargs)



    def __call__(self, model, x, y, N=100):
        '''
        Run the guess-n-check "optimization."
        '''

        # create a copy of the model
        model_copy = model.copy()

        # get the best fit parameters
        fitparams = self._opt_method(self._stat_method, model, x, y, N=N)
        model_copy.parameters = fitparams

        return model_copy

# write documentation that makes clear goodness must want to be minimized
# write wrapper that sets model bounds + fixed or not
# log axis for gof
# better default filename
# display mp4 in browswer



import numpy as np
import matplotlib.pyplot as plt
from matplotlib import colors

def star_plot(image,pos,ap_size=10,centroid=np.array([0,0]),r_in=20,r_out=30,back_plot=False,centroid_plot=False):


    '''
    Creates a plot of the image and the zoomed in images of the stars

    Parameters:
    -----------
    image : ndarray
        Full image of the frame
    pos : 2darray
        A list of positions in x,y pixel coordinates for each star.
        Needs to be as [[0,0],[1,1]...]
    ap_size : int
        The radius of a circular aperture in pixel size
    centroid : 2darray
        The centroid pixel location for each star to be plotted.
         (Will only plot if centroid_plot is True)
         Needs to be as [[0,0],[0,1]]
    r_in : int
        The inner radius of background aperture in pixel size
        (Ignore if back_plot=False)
    r_out : int
        The outer radius of background aperture in pixel size
        (Ignore if back_plot=False)
    back_plot : bool
        Set to True if want to plot the background aperture, False otherwise
    centroid_plot : bool
        Set to True if want the centroid location to be plotted, False otherwise

    Returns
    -------
    plots of the full image plus all the stars
    '''
    pos = np.array(pos)
    centroid =  np.array(centroid)
    #to get the number of plots needed
    num_plots = np.shape(pos)[0]

    plt.figure(figsize=[num_plots*10,5])
    plt.subplot(1,num_plots+1,1)
    plt.imshow(image)

    for i in range(num_plots):
        #i is essentially the star number
        x1 = pos[i,0]
        y1 = pos[i,1]
        if back_plot == True:
            #use the rout aperture size for cropping
            crop_image = image[int(x1-r_out):int(x1+r_out)+1,int(y1-r_out):int(y1+r_out)+1]
        elif back_plot == False:
            #use the aperture size plus 2 pixels for cropping
            crop_image = image[int(x1-ap_size-2):int(x1+ap_size+2),int(y1-ap_size-2):int(y1+ap_size+2)]
        plot_size = np.shape(crop_image)[0]
        plt.subplot(1,num_plots+1,i+2)
        plt.imshow(crop_image)
        if centroid_plot==True:
            m = plt.Circle((centroid[i,0]-x1+plot_size/2.-.5,centroid[i,1]-y1+plot_size/2.-.5),ap_size,facecolor='none',edgecolor='black')
        else:
            m = plt.Circle((plot_size/2.-.5,plot_size/2.-.5),ap_size,facecolor='none',edgecolor='black')
        plt.figure(1).add_subplot(1,num_plots+1,(i+2)).add_artist(m)
        if back_plot == True:
            if centroid_plot==True:
                m1 = plt.Circle((centroid[i,0]-x1+plot_size/2.-.5,centroid[i,1]-y1+plot_size/2.-.5),r_out,facecolor='none',edgecolor='cyan')
                m2 = plt.Circle((centroid[i,0]-x1+plot_size/2.-.5,centroid[i,1]-y1+plot_size/2.-.5),r_in,facecolor='none',edgecolor='cyan')
            else:
                m1 = plt.Circle((r_out,r_out),r_out,facecolor='none',edgecolor='cyan')
                m2 = plt.Circle((r_out,r_out),r_in,facecolor='none',edgecolor='cyan')
            plt.figure(1).add_subplot(1,num_plots+1,(i+2)).add_artist(m1)
            plt.figure(1).add_subplot(1,num_plots+1,(i+2)).add_artist(m2)

        if centroid_plot == True:
            m3 = plt.Circle((centroid[i,0]-x1+plot_size/2.-.5,centroid[i,1]-y1+plot_size/2.-.5),.2,facecolor='black',edgecolor='black')
            plt.figure(1).add_subplot(1,num_plots+1,(i+2)).add_artist(m3)
    plt.show()



from numpy import *
import pdb
import matplotlib.pyplot as plt

def mean_centroid(image, center, boxsize=60.):

	'''
    Determines the weighted mean centroid for multiple stars

    Parameters:
    -----------
    image : ndarray
        Image for aperture photometry to be performed on.
    center : 2darray
        A list of positions in x,y pixel coordinates for each star.
        Needs to be as [[0,0],[1,1]...]
		Basically is a best guess to the center
    boxsize : int, even number
        The size of the box to crop around star, needs to be even
    num_stars : int
        Number of stars to look at

    Returns
    -------
    centroid position for each star : array
    '''

	# figure out the number of stars from the list of centers
	num_stars = len(center)
	centroid = zeros((num_stars,2)) #setting up the centroid array
	L = boxsize #size of the box to isolate the star


	for k in range(num_stars):
		c = center[k] #get the approximate star location

		x = arange((c[0]-L/2.),(c[0]+L/2.),1)
		y = arange((c[1]-L/2.),(c[1]+L/2.),1)
		cropped_images = image[int(c[0]-L/2.):int(c[0]+L/2.),int(c[1]-L/2.):int(c[1]+L/2.)]
		cropped_images = where(isfinite(cropped_images),cropped_images,0)
		plt.imshow(cropped_images)
		plt.show()
		print k




		im = cropped_images
		Ii = []
		Jj = [] #keeps track of the row and column sums
		for i in range(int(L)):
			Ii.append(sum(im[i,:]))
			Jj.append(sum(im[:,i]))
		Ii=array(Ii)
		Jj=array(Jj)
		Imean = 1./L*sum(Ii)
		Jmean = 1./L*sum(Jj)

		Ii_red = Ii[where(Ii>Imean)]
		Jj_red = Jj[where(Jj>Jmean)]
		x_red = x[where(Ii>Imean)]
		y_red = y[where(Jj>Jmean)]
		xc = sum((Ii_red-Imean)*x_red)/sum(Ii_red-Imean)
		yc = sum((Jj_red-Jmean)*y_red)/sum(Jj_red-Jmean)
		x_center = xc #put back in normal coordinates
		y_center = yc

		centroid[k] = [x_center,y_center]

	return centroid



from ..imaging.visualize import *
from ..tpf import *

directory = 'examples'
mkdir(directory)

def test_animation(howmanydays=1.0):
    tpf = download_kepler_tpf()
    animate_tpf(tpf, howmanydays=howmanydays)



from ..imports import *
from .io import compile_rgb
import illumination as il
import astropy.units as u

def animate_tpf(tpf, showdifference=False, howmanydays=None, timestep=30):
    '''
    Create a .mp4 animation of the data in a TPF.

    Parameters
    ----------
    tpf : KeplerTargetPixelFile
        A time series of pixels.

    showdifference : bool
        Should we also show the difference image, of each frame
        with the median-frame subtracted?

    howmanydays : float
        The maximum number of days to animate.

    timestep : float
        The cadence at which to animate the movie, in units of minutes.
    '''

    # create a sequence of images
    seq = il.make_image_sequence(tpf.flux, time=tpf.time)

    # create a frame of the raw data
    frame = il.imshowFrame(data=seq, title='{} | {}'.format(tpf.mission, tpf.targetid))
    frames = [frame]


    if showdifference:
        # create a frame with the subtracted data
        diff = il.imshowFrame(data=seq, title='difference', processingsteps=['subtractmedian'])
        frames.append(diff)

    # create an illustration
    illustration = il.GenericIllustration(imshows=frames, sharecolorbar=False)
    filename = '{}-{}-animated.mp4'.format(tpf.mission, tpf.targetid)
    if howmanydays is None:
        maxtimespan = None
    else:
        maxtimespan = howmanydays*u.day
    if timestep is None:
        cadence = seq.cadence()
    else:
        cadence = timestep*u.minute
    illustration.animate(filename=filename, cadence=cadence, maxtimespan=maxtimespan)


def display_rgb(red=None, green=None, blue=None, size = 8, origin='upper'):
    '''
    Make a pediagogical plot showing how monochromatic red, green, and blue
    images can be combined to make a single color image.

    Parameters
    ----------
    red : 2D numpy array
        An image of the "red" brightness.
    green : 2D numpy array
        An image of the "green" brightness.
    blue : 2D numpy array
        An image of the "blue" brightness.
    '''

    # make sure that at least one of the images is defined
    if red is not None:
        anyimage = red
    elif green is not None:
        anyimage = green
    elif blue is not None:
        anyimage = blue
    assert(anyimage is not None)

    # replace undefined images with zeros
    if red is None:
        red = np.zeros_like(anyimage)
    if green is None:
        green = np.zeros_like(anyimage)
    if blue is None:
        blue = np.zeros_like(anyimage)

    # make sure the shapes of the images are consistent with each other
    assert(red.shape == green.shape)
    assert(blue.shape == green.shape)

    # set up the geometry for the figure
    aspectratio = red.shape[1]/float(red.shape[0])

    if aspectratio <= 1.0:
        figsize = (size*aspectratio, 5.0/3.0*size)
    else:
        figsize = (size, 5.0/3.0*size/aspectratio)
    plt.figure(figsize=figsize)

    # set up a grid of axes
    gs = plt.matplotlib.gridspec.GridSpec(3, 3, height_ratios=[1, 1, 3], wspace=0.01, hspace=0.01, left=0.1, right=0.9, bottom=0.05, top=0.9)
    ax = {}
    ax['color'] = plt.subplot(gs[2,:])

    colors = ['red', 'green', 'blue']
    for i, k in enumerate(colors):
        ax[k+'-gray'] = plt.subplot(gs[0,i], sharex=ax['color'], sharey=ax['color'])
        ax[k+'-mono'] = plt.subplot(gs[1,i], sharex=ax['color'], sharey=ax['color'])
    for k in ax:
        plt.sca(ax[k])
        plt.axis('off')

    for k in colors:
        ax[k+'-gray'].imshow(locals()[k], cmap=il.one2another('black', 'white'), origin=origin)
        ax[k+'-mono'].imshow(locals()[k], cmap=il.one2another('black', k), origin=origin)
        ax[k+'-gray'].set_title(k)

    ax['color'].imshow(compile_rgb(red,green,blue), origin=origin)



import numpy as np
import matplotlib.pyplot as plt
import photutils
import pdb
plt.ion()

def photometry(ax, image,pos,ap_size=10,r_in=20,r_out=30,back_photo=True):

    '''
    Create apertures around specified list of stars

    Parameters:
    -----------
    ax : axis class
        axis of a subplot.
    image : ndarray
        Image for aperture photometry to be performed on.
    pos : 2darray
        A list of positions in x,y pixel coordinates for each star.
        Needs to be as [[0,0],[1,1]...]
    ap_size : int
        The radius of a circular aperture in pixel size
    r_in : int
        The inner radius of background aperture in pixel size
        (Ignore if back_photo=False)
    r_out : int
        The outer radius of background aperture in pixel size
        (Ignore if back_photo=False)
    back_photo : bool
        Set to True if want to return an array of background values, False
        to ignore anything to do with background

    Returns
    -------
    photometry of each star : array
    average background pixel value around each star : array (If back_photo=True)
    plots image with the aperture and centroids located for each star
    '''

    nstars = np.arange(0,np.shape(pos)[0],1)
    name_stars = []
    for i in nstars:
        name_stars.append('Star {}'.format(i))
    #print name_stars

    aperture = photutils.CircularAperture(pos,r=ap_size)

    if back_photo == True:
        back_aperture = photutils.CircularAnnulus(pos,r_in,r_out)

    pos = np.array(pos)


    #plt.imshow(image,origin='lower')

    for i in range(len(name_stars)):
        circle1 = plt.Circle((pos[i,0], pos[i,1]), ap_size, color='black',fill=False,zorder=100)
        ax.add_artist(circle1)
        plt.axhline(pos[i,1],xmin=pos[i,0]/100.-.01,xmax=pos[i,0]/100.+.02,color='black')
        plt.axvline(pos[i,0],ymin=pos[i,1]/100.-.01,ymax=pos[i,1]/100.+.02,color='black')
        plt.text(pos[i,0]+ap_size+1, pos[i,1], name_stars[i],zorder=11)
        #print pos[i,1]
        #print pos[i,0]

        if back_photo == True:
            circle2 = plt.Circle((pos[i,0], pos[i,1]), r_in, color='cyan',fill=False,zorder=10)
            circle3 = plt.Circle((pos[i,0], pos[i,1]), r_out, color='cyan',fill=False,zorder=10)
            ax.add_artist(circle2)
            ax.add_artist(circle3)
    #aperture.plot(origin=(0,0),indices=None,ax=ax,fill=False)
    #plt.ion()




    phot_table = photutils.aperture_photometry(image,aperture)
    flux_values = phot_table['aperture_sum'].data #gets a list of the total flux in specified aperture for each star

    if back_photo == True:
        back_table = photutils.aperture_photometry(image,back_aperture)
        area = np.pi*(r_out**2-r_in**2)
        a = np.ones((np.shape(image)[0],np.shape(image)[1]))
        area = photutils.aperture_photometry(a,back_aperture)
        back_values = back_table['aperture_sum'].data/area['aperture_sum'].data
        return flux_values,back_values*np.pi*ap_size**2
    else:
        return flux_values



from .. import *

def test_batman():
    '''
    This function tests the batman transit light curve model.
    '''
    example_transit_model()

def test_plot(N=1e6, duration=10, cadence=1.0/60.0/24.0, period=3.14, t0=0.5, **kw):
    '''
    This function tests the plot_with_transit_model function.
    '''
    for e in [True, False]:
        lc = simulate_transit_data(N=N, duration=duration, cadence=cadence, period=period, t0=t0)
        plot_with_transit_model(lc, period=period, t0=t0, show_errors=e, **kw)

        folded = lc.fold(period=period, transit_midpoint=t0)
        plot_with_transit_model(folded, period=period, t0=t0, show_errors=e, **kw)
    return lc, folded


def test_plot_with_kepler():
    '''
    This function tests the plot_with_transit_model, using with Kepler data.
    '''

    # download a light curve
    lc = download_kepler_lc('Kepler-7')

    # phase fold it
    period = 4.8854892
    t0 = 2454967.27687
    transitparams = dict(b=0.5599, a=6.637, radius=0.08294)
    folded = lc.fold(period=period, transit_midpoint=bjd2bkjd(t0))

    # plot it
    plot_with_transit_model(folded, period=period, t0=t0, **transitparams)


def test_goodness(N=1e6, duration=10, cadence=1.0/60.0/24.0, period=3.14, t0=0.5, **kw):

    lc = simulate_transit_data(N=N, duration=duration, cadence=cadence, period=period, t0=t0,  **kw)
    folded = lc.fold(period=period, transit_midpoint=t0)
    return plot_with_transit_model(folded, period=period, t0=t0, goodness=chisq, **kw)



import illumination as il
from ..imports import *
from .apertures import *
from ..imaging import io
from tqdm import tqdm

from IPython.display import display

# is there a way to force %matplotlib notebook?
#from IPython import get_ipython
#get_ipython().run_line_magic('matplotlib', 'widget')

from astropy.table import vstack
import ipywidgets as widgets

from lightkurve import LightCurve

class Loupe(il.IllustrationBase):
    '''
    An interactive Illustration, that allows us to do
    aperture photometry on particular locations in an image.
    '''
    illustrationtype = 'Loupe'

    def create_frame(self, *args, **framekw):
        '''
        (part of __init__)
        Create the illustration frame.
        '''
        plt.ioff()

        # don't let this illustration talk
        self._pithy = True


        # figure out the geometry, for the figure
        aspectratio = self.image.shape[1]/float(self.image.shape[0])
        width = 5.5
        height = width/aspectratio
        #if aspectratio >=1:
        #    width = size*aspectratio
        #    height = size
        #else:
        #    width = size
        #    height = size/float(aspectratio)

        # create the basic illustration layout (this creates a self.grid)
        il.IllustrationBase.__init__(self, 1, 1,
                                  figkw=dict(figsize=(width, height), dpi=100),
                                  hspace=0.02, wspace=0.02,
                                  left=0.05, right=0.95,
                                  bottom=0.1, top=0.9)
        self._pithy = True

        # create an imshowFrame to display the image (and hide its text output)
        self.frames['imshow'] = il.imshowFrame(data=self.images, illustration=self, **framekw)
        self.frames['imshow']._pithy=True

        # create an ax for this frame
        self.frames['imshow'].ax = plt.subplot(self.grid[0, 0])


        # draw this illustration
        self.plot()

    def change_image(self, i, quick=False):

        # update the current image data
        self.image = self.images[i]

        # update the displayed image
        time = self.images.time[i]
        if not quick:
            self.update(time)
            plt.draw()

    def create_widgets(self, aperture_radius=5, subtract_background = False, background_radii=[15, 25], maxaperture=100, maxbackground=200, **kwargs):
        '''
        Create the control widgets for interactions.
        (part of __init__)

        Parameters
        ----------
        aperture_radius : int
            The default radius for the photometric aperture
            (can be changed interactively)

        subtract_background : bool
            Should we attempt to do background subtraction?
            (can be changed interactively)

        background_radii : list of two ints
            The default radii for background subtraction
            (can be changed interactively)

        maxaperture : int
            The maximum aperture radius to allow interactively.

        maxbackground : int
            The maximum background radius to allow interactively.
        '''

        ####################################
        # create the aperture radius control
        ####################################

        # a label for setting the aperture
        aperture_radius_label = widgets.Label('Photometric Aperture')

        # a slider to pick the radius of the aperture
        '''
        aperture_radius_slider = widgets.FloatSlider(
                                    value=aperture_radius,
                                    min=0,
                                    max=maxaperture,
                                    step=1,
                                    description='Radius',
                                    orientation='horizontal',
                                    continuous_update=False,
                                    readout=False,
                                    layout={'width':'100%'})
        '''
        aperture_radius_text = widgets.FloatText(
                                    value=aperture_radius,
                                    description='Radius',
                                    layout={'width':'80%'})

        #widgets.jslink([aperture_radius_slider, 'value'],
        #               [aperture_radius_text, 'value'])

        # make the combined aperture widget
        self._widget_aperture = widgets.VBox([aperture_radius_label,
                                              #aperture_radius_slider,
                                              aperture_radius_text])

        ###########################################
        # create the background subtraction control
        ###########################################

        # a checkbox to decide if we should subtract the background
        background_checkbox = widgets.Checkbox(
                                        value=False,
                                        description='Subtract Background?',
                                        disabled=False)

        # a label for the background subtraction
        background_label = widgets.Label('Background Subtraction',
                                         disabled=True)

        # create text boxes for the inner and outer radii
        background_inner_text = widgets.BoundedFloatText(
                                    value=background_radii[0],
                                    description='Inner Radius',
                                    layout={'width':'80%'})

        background_outer_text = widgets.BoundedFloatText(
                                    value=background_radii[1],
                                    description='Outer Radius',
                                    layout={'width':'80%'})

        widgets.jslink((aperture_radius_text, 'value'),
                       (background_inner_text, 'min'))

        widgets.jslink((background_inner_text, 'value'),
                       (background_outer_text, 'min'))

        # a range slider to pick the inner + outer radii of the annulus
        background_slider = widgets.IntRangeSlider(
                                value=background_radii,
                                min=0,
                                max=maxbackground,
                                step=1,
                                description='Radii',
                                disabled=True,
                                continuous_update=False,
                                orientation='horizontal',
                                readout=True,
                                layout={'width':'100%'})

        # only turn on the background slider if we need it
        def update_background_slider(checkbox):
            '''
            What do we do to the background slider
            when the background checkbox is updated?
            '''
            do_background = checkbox['new'] == False
            background_inner_text.disabled = do_background
            background_outer_text.disabled = do_background

        background_checkbox.observe(update_background_slider, names='value')

        # don't let the background inner radius go inside the aperture radius
        #widgets.jslink((aperture_radius_slider, 'value'),
        #               (background_slider, 'min'))

        # create the combined background widget
        self._widget_background = widgets.VBox([background_label,
                                               background_checkbox,
                                               background_inner_text,
                                               background_outer_text])


        #########################################
        # create some output widgest for displays
        #########################################

        # create a place to share instructions
        self._widget_instruction = widgets.Output(layout={'border': '1px solid black',
                                                        'margin': '1% 0% 1% 0%',
                                                        'max_height':'30px',
                                                        'background_color':'red',
                                                        })

        # create an output to share results
        self._widget_results = widgets.Output(layout={'border': '1px solid black',
                                                    'margin': '1% 0% 1% 0%',
                                                    'max_height':'200px',
                                                    'min_height':'30px'})

        #####################################
        # create time slider to change images
        #####################################

        # a slider to pick the radius of the aperture
        image_slider = widgets.IntSlider(
                                    value=0,
                                    min=0,
                                    max=len(self.images)-1,
                                    step=1,
                                    description='#',
                                    orientation='vertical',
                                    continuous_update=False,
                                    readout=True,
                                    layout=dict(height='100%'))

        # create a container to store the time slider in
        self._widget_imageslider = widgets.VBox([image_slider],
                             layout=dict(margin='0% 0% 0% 0%'))


        # set up the interaction loop for the photometry
        self._interaction_photometry = widgets.interactive(self.photometry,
                                         aperture_radius=aperture_radius_text,
                                         background_inner=background_inner_text,
                                         background_outer=background_outer_text,
                                         subtract_background=background_checkbox,
                                         image_number=image_slider)


        # create a shared widget for the controls
        self._widget_controls = widgets.VBox([self._widget_aperture, self._widget_background])

    def __init__(self, images, aperture_radius=5, subtract_background = False, background_radii=[15, 25], **framekw):
        '''
        Parameters
        ----------

        image : 2D array, str
            An image or a path to a filename containing an image.

        **framekw passed to CameraFrame
        '''

        ####################################
        # set the defaults
        ####################################

        self.aperture_radius = aperture_radius
        self.subtract_background = subtract_background
        self.background_radii = background_radii


        ####################################
        # load sequence of images
        ####################################

        # load an image if given a filename
        if type(images) == str:
            images = io.read_image(images)

        self.images = il.make_image_sequence(images)

        #  define the first image, initially
        self.image = self.images[0]


        # create the widgets
        self.create_widgets(aperture_radius=aperture_radius,
                            subtract_background=subtract_background,
                            background_radii=background_radii)

        with self._widget_instruction:
            print("Please [a]dd or [z]ap an aperture at the cursor location.")


        # populate the illustration with data
        self.create_frame()


        self._widget_image = widgets.VBox([self.figure.canvas],
                             layout=dict(width='62%', margin='0% 2% 0% 0%'))

        self._widget_left = widgets.VBox([
                                          self._widget_controls,
                                          self._widget_results],
                            layout=dict(width='30%', margin='0% 0% 0% 2%'))


        self._widget_interaction = widgets.HBox([self._widget_left,
                                            self._widget_imageslider,
                                            self._widget_image])


        self._widget_layout = widgets.VBox([self._widget_instruction,
                                            self._widget_interaction])



        display(self._widget_layout)

        # set it up so we can add or subtract an aperture by clicking on the image
        self.figure.canvas.mpl_connect('key_press_event', self.do_something_with_keyboard)


        # start with no apertures defined
        self.apertures = []
        self.napertures = 0

    def do_something_with_keyboard(self, event):
        '''
        If the user clicks on the imshow, do something with that click.

        Parameters
        ----------
        event : matplotlib.backend_bases.LocationEvent
        '''

        # store the last event that was clicked
        self.lastevent = event

        # define the available options
        self.options = {'z':self.remove_aperture,
                        'a':self.add_aperture}

        # do the requested action
        key_pressed = event.key.lower()
        action = self.options.get(key_pressed, None)
        if action is not None:
            action(event.xdata, event.ydata)

            # update the plot and the photometry
            plt.draw()
            self._interaction_photometry.update()

    def add_aperture(self, x, y):
        '''
        Add an aperture at a particular (x, y) location.
        '''
        #with self.out:
        #    print('adding aperture at {}'.format((x,y)))
        position = np.array([x,y])
        name = '{}'.format(self.napertures)

        new = InteractiveAperture(name=name,
                                  pos=position,
                                  loupe=self,
                                  aperture_radius=self.aperture_radius,
                                  subtract_background=self.subtract_background,
                                  background_radii=self.background_radii,
                                  )

        # plot this aperture on the image
        new.plot(ax=self.frames['imshow'].ax)

        # store this new aperture in a list
        self.apertures.append(new)
        self.napertures += 1

    def remove_aperture(self, x, y):
        '''
        Remove the closest aperture to the given (x,y) location.

        Parameters
        ----------
        x, y : float
            The location where we want to remove an aperture.
        '''

        positions = [a.positions[0] for a in self.apertures]
        xexisting, yexisting = np.array(positions).T
        distance = (xexisting - x)**2 + (yexisting - y)**2
        closest = np.argmin(distance)

        toremove = self.apertures.pop(closest)
        toremove.erase()


    def photometry(self, aperture_radius=5,
                         subtract_background = False,
                         background_inner=15,
                         background_outer=25,
                         image_number=0,
                         quick=False):

        '''
        Create apertures around specified list of stars

        Parameters:
        -----------

        aperture_radius : float
            The radius of a circular aperture in pixel size
        r_in : float
            The inner radius of background aperture in pixel size
            (Ignore if back_photo=False)
        r_out : float
            The outer radius of background aperture in pixel size
            (Ignore if back_photo=False)

        Returns
        -------
        photometry of each star : array
        average background pixel value around each star : array (If back_photo=True)
        plots image with the aperture and centroids located for each star
        '''


        background_radii=[background_inner, background_outer]
        # point at this current figure
        if not quick:
            plt.figure(self.figure.number)

        # set the image to image
        self.change_image(image_number, quick=quick)

        # if we have no apertures, do nothing
        if len(self.apertures) == 0:
            return None

        # pull out the plotting ax and the image
        ax = self.frames['imshow'].ax
        image = self.image

        self.aperture_radius = aperture_radius
        self.subtract_background = subtract_background
        self.background_radii = background_radii

        for i, a in enumerate(self.apertures):
            a.update()

        #if back_photo == True:
        #    back_table = photutils.aperture_photometry(image,back_aperture)
        #    area = np.pi*(r_out**2-r_in**2)
        #    a = np.ones((np.shape(image)[0],np.shape(image)[1]))
        #    area = photutils.aperture_photometry(a,back_aperture)
        #    back_values = back_table['aperture_sum'].data/area['aperture_sum'].data
        #    return flux_values,back_values*np.pi*aperture_radius**2
        #else:

        self.measurements = vstack([a.table for a in self.apertures])

        if not quick:
            self._widget_results.clear_output()
            with self._widget_results:
                print('r={:.0f}px (area={:.1f}px)'.format(self.aperture_radius, self.apertures[0].area()))
                print(self.measurements)
            plt.draw()

        return self.measurements

    def make_lightcurves(self):

        # create an empty list of photometry tables and times
        tables = []
        times = []

        # loop over all the images
        for i in tqdm(range(len(self.images))):
            table = self.photometry(image_number=i, quick=True)
            table.add_index('#')
            tables.append(table)
            times.append(self.images.time[i].jd)

        lightcurves = {}
        for k in tables[0]['#']:
            lightcurves[k] = LightCurve( flux=[t.loc[k]['flux'] for t in tables])

        self.lightcurves = lightcurves
        return lightcurves



from lightkurve import KeplerTargetPixelFile, TessTargetPixelFile, search_targetpixelfile
from .imports import *
from .tools import *

def download_kepler_tpf(star='Kepler-186',
                        quarter=1,
                        cadence='long',
                        **kw):
    '''
    This function is a wrapper to download one or more "Target Pixel Files" (TPF)
    of Kepler data. Each TPF is basically a movie; it contains a time series
    of images in a tiny little area around a particular Kepler target.

    This has been tested only on data from the main Kepler mission and K2.

    Parameters
    ----------

    star: str, int
        The name of the star whose light curve you want to download.
        This can be a name ("Kepler-42", "KOI-961") or a KIC number
        (8561063)

    quarter: str, int
        Which Kepler quarter to download?
            1, 2, 3 ... = download a specific quarter

    cadence: str
        'long' for Kepler long cadence
        'short' for Kepler short cadence

    Returns
    -------

    tpf: KeplerTargetPixelFile object
        This is a `lightkurve`-style TargetPixelFile object, which contains
        the attributes `tpf.time` (times in JD) and `tpf.flux` (the brightness
        of each pixel as a function of time), as well as lots of methods for
        analysis and plotting.
    '''

    # download a KeplerLightCurveFile from the MAST archive
    tpf = search_targetpixelfile(star,
                                 quarter=quarter,
                                 cadence=cadence,
                                 **kw).download()
    return tpf

def download_k2_tpf(star='WASP-47', **kw):
    '''
    This function is a wrapper a "Target Pixel Files" (TPF) for Kepler/K2 data.
    Each TPF is basically a movie; it contains a time series of images in a tiny
    little area around a particular Kepler/K2 target.

    This has been tested only on data from the main Kepler mission and K2.

    Parameters
    ----------

    star: str, int
        The name of the star whose light curve you want to download.
        This can be a name ("K2-18", "TRAPPIST-1") or an EPIC number
        (246199087).

    Returns
    -------

    tpf: KeplerTargetPixelFile object
        This is a `lightkurve`-style TargetPixelFile object, which contains
        the attributes `tpf.time` (times in JD) and `tpf.flux` (the brightness
        of each pixel as a function of time), as well as lots of methods for
        analysis and plotting.
    '''

    # download a KeplerLightCurveFile from the MAST archive
    tpf = search_targetpixelfile(star, **kw).download()
    return tpf

def download_tess_tpf(star=261136679, **kw):
    '''
    This function is a wrapper a "Target Pixel Files" (TPF) for TESS data, as
    accessed through the MAST archive. Each TPF is basically a movie; it
    contains a time series of images in a tiny little area around a
    particular TESS target.

    This has been tested only on data from TESS Sector 1. The available
    data are described here:

    https://archive.stsci.edu/prepds/tess-data-alerts/

    Parameters
    ----------

    star: str, int
        The TESS Input Catalog ID of the star whose light curve
        you want to download. (We should make a more flexible interface
        for this in the future, once more data are available).

    Returns
    -------

    tpf: TessTargetPixelFile object
        This is a `lightkurve`-style TargetPixelFile object, which contains
        the attributes `tpf.time` (times in JD) and `tpf.flux` (the brightness
        of each pixel as a function of time), as well as lots of methods for
        analysis and plotting.
    '''

    # download a TessTargetPixelFile from the MAST archive
    url = "https://archive.stsci.edu/hlsps/tess-data-alerts/hlsp_tess-data-alerts_tess_phot_{:011}-s01_tess_v1_tp.fits".format(int(star))
    tpf = TessTargetPixelFile(url, **kw)

    # return the tpf
    return tpf

tpf = ()



from .. import *

def test_trapezoid():
    '''
    This tests a few different astropy trapezoid transit models.
    '''
    t = np.linspace(-1.5, 1.5, 10000)

    for tau in np.arange(3)*0.12:
        for T in np.arange(3)*0.1:
            model = TrapezoidTransit(T=T, tau=tau)
            plt.plot(t, model(t), label=repr(model))
    plt.legend(fontsize=4)
    plt.show()


def test_batman(N=1e6, duration=10, cadence=1.0/60.0/24.0, period=3.14, t0=0.3, **kw):
    '''
    This tests if the batman astropy model works.
    '''
    lc = simulate_transit_data(N=N, duration=duration, cadence=cadence, period=period, t0=t0, **kw)
    ba = BatmanTransit(period=period, t0=t0, **kw)
    lc.scatter()
    plt.plot(lc.time, ba(lc.time), label=repr(ba), color='mediumseagreen')
    plt.legend(fontsize=6)


def test_batman(N=1e6, duration=10, cadence=1.0/60.0/24.0, period=3.14, t0=0.3, **kw):
    '''
    This tests if the batman astropy model works.
    '''
    lc = simulate_transit_data(N=N, duration=duration, cadence=cadence, period=period, t0=t0, **kw)
    ba = BatmanTransit(period=period, t0=t0, **kw)
    lc.scatter()
    plt.plot(lc.time, ba(lc.time), label=repr(ba), color='mediumseagreen')
    plt.legend(fontsize=6)



from ..statistics import *

def test_photons():
    '''
    This function tests the toy photon light curve.
    '''

    lc = create_photon_lightcurve()
    lc.plot(marker='.', linewidth=0)

def test_bucket():
    '''
    This function tests the catch_photons_in_bucket function.
    '''
    N = catch_photons_in_bucket(rate=1, time=2, diameter=3, visualize=False)
    print(N)



from ..lightcurves import *
from ..modeling import *

def test_download_kepler_lc():
    '''
    This function tests the wrapper for downloading
    a single quarter of Kepler data.
    '''

    lc = download_kepler_lc('Kepler-17')
    ax = lc.plot(marker='.', linewidth=0)
    singlequarter = download_kepler_lc('Kepler-17', quarter=1)
    singlequarter.plot(ax=ax, marker='.', linewidth=0)


    folded = lc.fold(1.48571092)
    folded.scatter(marker='.', linewidth=0)

def test_locate():
    '''
    This function should draw a single circle
    around every transit in the Kepler-17 light curve.
    '''

    # download a Kepler-17 light curve
    lc = download_kepler_lc('Kepler-10', quarter=1)

    # make a light curve plot
    lc.flatten().plot(marker='.', linewidth=0)

    # draw lines for the planet on top
    locations = locate_transits(lc, period=0.837491, t0=2454964.57513, color='royalblue', name='Kepler-10b')
    locations = locate_transits(lc, period=45.294301, t0=2455062.26648, color='darkorange', name='Kepler-10c')

    plt.legend(bbox_to_anchor=(1,1), loc='upper left')
    plt.tight_layout()

def test_extract(period = 1.234, t0 = 2451234.5678, window = 0.05):

    # create a simulated light curve
    lc = simulate_transit_data(period=period, t0=t0)

    # split into two light curves, one with transits, one without
    transits, notransits = extract_transits(lc, period, epoch=t0, window=window)

    # make sure the sizes of these light curves make sense
    assert(len(transits.flux) + len(notransits.flux) == len(lc.flux))

    # make sure we can plot both of them (and)
    ax = transits.scatter(c='gray',normalize=False)
    notransits.scatter(ax=ax, c='darkorange',normalize=False)

    return transits, notransits



import numpy as np

# for Python 2/3 compatibility
try:
    input = raw_input
except NameError:
    pass

def sumofsquares(residuals):
    '''
    This calculates a goodness-of-fit from an array of residuals.
    (a lower value implies a better fit)

    Parameters
    ----------
    residuals : `~numpy.ndarray`
        Array of values for (data-model)/sigma

    Returns
    -------
    gof : float
        A single goodness-of-fit metric
        (in this case, sum of squares)
    '''
    return np.sum(residuals**2)

def mean(residuals):
    '''
    This is a goodness-of-fit function. It calculate the mean
    of the residuals.

    Parameters
    ----------
    residuals : array
        Array of residuals = (data - model)/sigma

    Returns
    -------
    mean : float
        The mean of the residuals.
    '''

    return np.mean(residuals)

def chisq(residuals):
    '''
    This is a goodness-of-fit function. It calculates the sum of the squares
    of the residuals. This is commonly called "chi squared".

    Parameters
    ----------
    residuals : array
        Array of residuals = (data - model)/sigma

    Returns
    -------
    chisq : float
        The sum of the squared residuals.
    '''

    return np.sum(residuals**2)

def votes(residuals=None):
    '''
    This is a goodness-of-fit function. It averages votes
    from user input, and then spits out a 1-10 score.

    Returns
    -------
    averagevote : float
        The average of a bunch of user-entered votes.
    '''

    # get input from the user
    answer = input('On a scale of 1-10, how good is this model?\n[separate multiple entries by commas]\n')

    # convert a string into numbers, and return its average
    average = np.mean([float(x) for x in answer.split(',')])

    # return that average out of the function
    return average
